

import logging; logger = logging.getLogger(__name__)



from .controls.control_base import ControlBase
from .controls.control_boundingslider import ControlBoundingSlider
from .controls.control_button import ControlButton
from .controls.control_checkbox import ControlCheckBox
from .controls.control_checkboxlist import ControlCheckBoxList
try:
    from .controls.control_codeeditor import ControlCodeEditor
except Exception as e:
    logger.warning('ControlCodeEditor will not work. Please check QScintilla is installed.')
    logger.debug('ControlCodeEditor will not work', exc_info=True)
from .controls.control_combo import ControlCombo
from .controls.control_dir import ControlDir
from .controls.control_dockwidget import ControlDockWidget
from .controls.control_emptywidget import ControlEmptyWidget
from .controls.control_file import ControlFile
from .controls.control_filestree import ControlFilesTree
try:
    from .controls.control_image import ControlImage
except Exception as e:
    logger.warning('ControlImage will not work. Please check OpenCV is installed.')
    logger.debug('ControlImage will not work', exc_info=True)
from .controls.control_label import ControlLabel
from .controls.control_list import ControlList
from .controls.control_matplotlib import ControlMatplotlib
from .controls.control_mdiarea import ControlMdiArea
from .controls.control_number import ControlNumber
try:
    from .controls.control_opengl import ControlOpenGL
except Exception as e:
    logger.warning('ControlOpenGL will not work. Please check PyOpenGL is installed.')
    logger.debug('ControlOpenGL will not work', exc_info=True)
from .controls.control_progress import ControlProgress
from .controls.control_slider import ControlSlider
from .controls.control_tableview import ControlTableView
from .controls.control_text import ControlText
from .controls.control_password import ControlPassword
from .controls.control_textarea import ControlTextArea
from .controls.control_toolbox import ControlToolBox
from .controls.control_toolbutton import ControlToolButton
from .controls.control_tree import ControlTree
from .controls.control_treeview import ControlTreeView
from .controls.control_visvis import ControlVisVis
from .controls.control_visvisvolume import ControlVisVisVolume
try:
    from .controls.control_web import ControlWeb
except Exception as e:
    logger.warning('ControlWeb will not work. Please check PyQt5.QtWebEngineWidgets is installed.')
    logger.debug('ControlWeb will not work', exc_info=True)
from .controls.control_event_timeline.control_eventtimeline import ControlEventTimeline
from .controls.control_events_graph.control_eventsgraph import ControlEventsGraph
try:
    from .controls.control_player.control_player import ControlPlayer
except Exception as e:
    logger.error( e )


#!/usr/bin/python
# -*- coding: utf-8 -*-

""" pyforms_gui.controls.ControlPlayer.VideoGLWidget

"""

from pyforms_gui.controls.control_player.AbstractGLWidget import AbstractGLWidget
from AnyQt.QtOpenGL import QGLWidget

class VideoGLWidget(AbstractGLWidget, QGLWidget): pass


#!/usr/bin/python
# -*- coding: utf-8 -*-


__author__      = "Ricardo Ribeiro"
__credits__     = ["Ricardo Ribeiro"]
__license__     = "MIT"
__version__     = "0.0"
__maintainer__  = "Ricardo Ribeiro"
__email__       = "ricardojvr@gmail.com"
__status__      = "Development"

from AnyQt.QtCore import QMargins
from AnyQt.QtWidgets import QWidget, QVBoxLayout
from pyforms_gui.controls.control_base import ControlBase
from matplotlib.figure import Figure
from AnyQt import _api

if _api.USED_API == _api.QT_API_PYQT5:
    from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg as FigureCanvas
    from matplotlib.backends.backend_qt5agg import NavigationToolbar2QT as NavigationToolbar
elif _api.USED_API == _api.QT_API_PYQT4:
    from matplotlib.backends.backend_qt4agg import FigureCanvasQTAgg as FigureCanvas
    from matplotlib.backends.backend_qt4agg import NavigationToolbar2QT as NavigationToolbar

class ControlMatplotlib(ControlBase, QWidget):

    def __init__(self, *args, **kwargs):
        QWidget.__init__(self)
        ControlBase.__init__(self, *args, **kwargs)

        self.on_draw = kwargs.get('on_draw', self.on_draw) 

    def init_form(self):

        self._fig   = Figure((5.0, 4.0), dpi=100)
        self.canvas = FigureCanvas(self._fig)
        self.canvas.setParent(self)
        self.mpl_toolbar = NavigationToolbar(self.canvas, self)
     
        vbox = QVBoxLayout()
        vbox.addWidget(self.canvas)
        vbox.addWidget(self.mpl_toolbar)
        vbox.setContentsMargins(QMargins(0,0,0,0))
        self.setLayout(vbox)
        super(ControlMatplotlib, self).init_form()

    @property
    def value(self): return None

    @value.setter
    def value(self, value): 
        self.on_draw = value
        self.draw()

    def draw(self): 
        self.on_draw(self._fig)
        self.canvas.draw()

    def on_draw(self, figure):
        """ Redraws the figure
        """
        x = range(len(self.value))

        #self._axes = self._fig.add_subplot(111)
        
        #self._axes.bar(left=x, height=self.data)
        #self.canvas.draw()

        axes = figure.add_subplot(111, projection='3d')
        axes.clear(); 
        pts = axes.scatter(x, x, x, c=x)
        figure.colorbar(pts)


    ############################################################################
    ############ Properties ####################################################
    ############################################################################

    @property
    def fig(self): return self._fig
    @fig.setter
    def fig(self, value): self._fig = value


    @property
    def form(self): return self


# !/usr/bin/python
# -*- coding: utf-8 -*-

import sys
from confapp      import conf
from AnyQt.QtWidgets import QWidget
from AnyQt.QtGui 	 import QColor, QPainter, QFont
from AnyQt 		     import QtCore

from pyforms_gui.controls.control_events_graph.Track 	 import Track
from pyforms_gui.controls.control_events_graph.Pointer import Pointer
from pyforms_gui.controls.control_events_graph.Event 	 import Event


class EventsWidget(QWidget):
	"""
	Timeline widget definition to be used in the ControlEventTimeline
	"""

	_defautcolor = QColor(100, 100, 255)

	def __init__(self, scroll):
		"""
		
		:param scroll: 
		"""
		self._is_painting = False
		self._break_draw = False  # This variable indicates if should continuing drawing the widget or should move foward

		super(EventsWidget, self).__init__()
		self._scroll = scroll

		# Set the widget background to white ############################
		palette = self.palette()
		palette.setColor(self.backgroundRole(), QtCore.Qt.white)
		self.setPalette(palette)
		#################################################################

		self._max_time = 0
		self._tracks = []  # List of tracks
		self._pointer = Pointer(0, self, scroll)  # Timeline ( greenline )
		self.tracks_height = 30  # Height in pixels
		self.scale = conf.PYFORMS_CONTROL_EVENTS_GRAPH_DEFAULT_SCALE

	##########################################################################
	#### HELPERS/FUNCTIONS ###################################################
	##########################################################################

	def clear(self):
		"""
		Clean all events
		"""
		for track in self._tracks:
			track.clear()
		del self._tracks[:]
		self._tracks = []
		self._break_draw = True and self._is_painting
		self.repaint()

	def add_track(self, title=None):
		"""
		Adds a new track
		:param title: 
		:return: track added 
		"""
		self._tracks.append(Track(parent=self))
		self.setMinimumHeight(self.which_top(len(self._tracks)))
		if title: self._tracks[-1].title = title
		return self._tracks[-1]

	def add_event(self, begin, end, title='', track=0, color="#FFFF00"):
		"""
		Adds an event.		
		:param begin: 
		:param end: 
		:param title: 
		:param track: 
		:param color: 
		:return: 
		"""

		period = Event(begin, end,
		               title=title,
		               parentWidget=self,
		               color=color
		               )

		if self._max_time < end:
			self._max_time = end
		# Create new tracks in case the variable track does not exists
		if len(self._tracks) <= track:
			for i in range(len(self._tracks), track + 1):
				t = self.add_track()
				t.title = title
		#################################################

		end_pixel = end / self._scale
		if end_pixel > self._scroll.maximum():
			self._scroll.setMaximum(end_pixel)

		self._tracks[track].add_event(period)
		return period

	def __check_current_time_is_visible(self, current_time):
		"""
		This function check if the current_time is visible to the user.
		:param current_time: 
		"""
		scroll_limit = (self._scroll.sliderPosition() + self.width()) * self._scale

		if current_time > scroll_limit:
			self._scroll.setMaximum(current_time / self._scale)
			self._scroll.setSliderPosition(current_time / self._scale - self.width())
		if current_time < (self._scroll.sliderPosition() * self._scale):
			self._scroll.setSliderPosition(current_time / self._scale)

	def which_track(self, y):
		"""
		Return the track index which collide with the Y coordenate
		:param y: 
		:return: 
		"""
		return (y - 20) // self.tracks_height

	def which_top(self, track):
		"""
		Return the Y coordenate where the track index start
		:param track: 
		:return: 
		"""
		return track * self.tracks_height + 20

	##########################################################################
	#### DRAW FUNCTIONS ######################################################
	##########################################################################

	def paintEvent(self, e):
		"""
		
		:param e: 
		:return: 
		"""
		self._is_painting = True
		super(EventsWidget, self).paintEvent(e)

		painter = QPainter()
		painter.begin(self)
		painter.setRenderHint(QPainter.Antialiasing)
		painter.setFont(QFont('Decorative', 8))

		slider_pos = self._scroll.sliderPosition()
		start = slider_pos * self._scale
		end = start + self.width() * self._scale

		for i, track in enumerate(self._tracks):
			if self._break_draw:
				break
			track.draw_events(painter, start, end, track_index=i, left_shift=-self._scroll.sliderPosition(),
			                   scale=self._scale)

		# Draw only from pixel start to end
		painter.setPen(QtCore.Qt.DashLine)
		painter.setOpacity(0.3)

		# print('Draw', start, end, self._scale, self._scroll.sliderPosition(), self.width())

		# Draw vertical lines
		for x in range(start - (start % (100 * self._scale)), end, 100 * self._scale):
			x2draw = (x - slider_pos * self._scale) // self._scale
			painter.drawLine(x2draw, 20, x2draw, self.height())
			string = str(x)
			boundtext = painter.boundingRect(QtCore.QRectF(), string)
			painter.drawText(x2draw - boundtext.width() / 2, 15, string)

		for index, track in enumerate(self._tracks):
			top = self.which_top(index)
			# print(top)
			painter.drawLine(0, top, self.width(), top)
			painter.drawText(10, top + 15, track.title)
		painter.setOpacity(1.0)

		self._pointer.draw(painter, left_shift=-slider_pos, scale=self._scale)  # Draw the time pointer
		painter.end()

		self._break_draw = False
		self._is_painting = False

	##########################################################################
	#### PROPERTIES ##########################################################
	##########################################################################

	@property
	def scale(self):
		return self._scale

	@scale.setter
	def scale(self, value):
		self._scale = value
		self._scroll.setMaximum(self._max_time / self._scale)

	@property
	def scroll(self):
		return self._scroll

	@property
	def position(self):
		return self._pointer._position

	@position.setter
	def position(self, position):
		self._pointer._position = position
		self._break_draw = True and self._is_painting

		if (position / self._scale) >= sys.maxsize:
			self.scale += 1

		#######################################################################
		# Check if the current time position is inside the scroll
		# if is not in, update the scroll position
		self.__check_current_time_is_visible(position)
		#######################################################################
		self.repaint()

	@property
	def color(self):
		return self._defautcolor

	@color.setter
	def color(self, value):
		self._defautcolor = value

	@property
	def tracks_height(self):
		return self._tracks_height

	@tracks_height.setter
	def tracks_height(self, value):
		self._tracks_height = value
		new_height = len(self.tracks) * value + 20
		if new_height > self.height():
			self.setMinimumHeight(new_height)
		self._break_draw = True and self._is_painting
		self.repaint()

	@property
	def tracks(self):
		"""
		Get list of tracks
		:return: 
		"""
		return self._tracks

	##########################################################################
	#### IMPORT AND EXPORT DATA ##############################################
	##########################################################################

	def import_csv(self, csvfileobject):
		"""
		Extracts info from a file object and stores it in memory in
		order to display it on the timeline.

		Refer to the `export` method to learn about input file format
		and structure.
		"""
		# Clear previously stored info
		self._tracks = []
		self._selected = None

		for row in csvfileobject:
			if row[0] == "T":
				track = self.add_track()
				track.properties = row
			elif row[0] == "P":
				event = self.add_event(0, 1, '-')
				event.properties = row

	def export_csv(self, csvfileobject):
		"""
		Processes all timeline data in an arranged format to be written
		in a CSV file.


		Current file structure:
		=======================

		--- CSV FILE BEGIN ---
		Track info line
		Event info line
		Event info line
		...
		Track info line
		Event info line
		Event info line
		...
		Track info line
		Event info line
		Event info line
		...
		--- CSV FILE END ---


		Track info line format:
		=======================

		| T | Total # of events in this track |  |  | Color | Label |


		Event info line format:
		=======================

		| P | Lock status | Begin frame | End frame | Comment | Color |  |
		"""
		for index, track in enumerate(self._tracks):
			csvfileobject.writerow(track.properties)
			for delta in track.events:
				csvfileobject.writerow(delta.properties)



# !/usr/bin/python3
# -*- coding: utf-8 -*-

import re
import logging
from confapp import conf

logger = logging.getLogger(__name__)


class PluginsFinder(object):
    """
    Manage and find plugins at runtime
    """

    def __init__(self, plugins=[]):
        self._plugins = plugins

    def __add__(self, other):
        self._plugins.append(other)
        return self

    def __sub__(self, other):
        self._plugins.remove(other)
        return self

    def find_class(self, class_full_name):
        res = self.get_module_and_class(class_full_name)
        if res is None:
            return []

        package_name, class_name = res

        res = []
        for plugin in self._plugins:
            try:
                values = class_full_name.split('.')

                class_name = values[-1]
                module_name = ('.' + '.'.join(values[:-1])) if len(values) > 1 else ''
                module = __import__(plugin + module_name, fromlist=[''])
                try:
                    class_def = getattr(module, class_name)
                    res.append(class_def)
                    logger.debug("FOUND: {0:20} IN PLUGIN:   {1:50}".format(package_name, plugin))

                except AttributeError:
                    if not conf.PYFORMS_SILENT_PLUGINS_FINDER:
                        logger.debug('Importing setting {0} {1} {2}'.format(str(plugin), str(package_name), str(class_name)),  exc_info=True)
                        
            except ImportError:
                if not conf.PYFORMS_SILENT_PLUGINS_FINDER:
                    logger.debug('Importing setting {0} {1} {2}'.format(str(plugin), str(package_name), str(class_name)),  exc_info=True)

            except:
                logger.debug(
                    'Importing model {0} {1} {2}'.format(str(plugin), str(package_name), str(class_name)),
                    exc_info=True)
                pass

        return res

    def get_module_and_class(self, plugin_info):
        """
        :param plugin_info:
        :return:
        """
        expression = re.compile('(.*)\.(\w*)')

        res = re.findall(expression, plugin_info)
        if len(res) > 0:
            matches = re.findall(expression, plugin_info)[0]
            plugin_module = matches[0]
            plugin_class = matches[1]
            return plugin_module, plugin_class
        else:
            return None



import csv
from pyforms_gui.basewidget import BaseWidget
from pyforms_gui.allcontrols import ControlText
from pyforms_gui.allcontrols import ControlButton
from pyforms_gui.allcontrols import ControlFile
from pyforms_gui.allcontrols import ControlList
from pyforms_gui.allcontrols import ControlNumber


class CsvParserDialog(BaseWidget):

    def __init__(self, parent=None):
        super(CsvParserDialog, self).__init__('CSV Choose the columns', parent_win = parent)
        self._filename = None

        # Definition of the forms fields
        self._filename = ControlFile('CSV File')
        self._separator = ControlText('Separator', default=';')
        self._startingrow = ControlNumber('Starting row', default=0)
        self._frameCol = ControlNumber('Frame column', default=0, minumum=0, maximum=100)
        self._xCol = ControlNumber('X column', default=1, minumum=0, maximum=100)
        self._yCol = ControlNumber('Y column', default=2, minumum=0, maximum=100)
        self._zCol = ControlNumber('Z column', default=3, minumum=0, maximum=100)
        self._filePreview = ControlList('Preview')
        self._loadButton = ControlButton('Load')

        self._formset = [('_filename','_startingrow'), ('_separator', '_frameCol', '_xCol', '_yCol', '_zCol', '_loadButton'), '_filePreview']
        self._separator.changed_event = self.__refreshPreview
        self._filename.changed_event  = self.__refreshPreview
        self._startingrow.changed_event = self.__refreshPreview

        #self._filename.value = '/home/ricardo/Downloads/2012.12.01_13.48_3D_POSITIONS_version_03.06.2015.csv'

    @property
    def filename(self): return self._filename.value

    @filename.setter
    def filename(self, value):
        self._filename.value = value
        self.__refreshPreview()

    @property
    def load_file_event(self): return self._loadButton.value

    @load_file_event.setter
    def load_file_event(self, value): self._loadButton.value = value

    @property
    def separator(self): return self._separator.value

    @property
    def starting_row(self): return self._startingrow.value
    @starting_row.setter
    def starting_row(self, value): self._startingrow.value = value

    @property
    def frameColumn(self): return self._frameCol.value
    @frameColumn.setter
    def frameColumn(self, value): self._frameCol.value = value

    @property
    def xColumn(self): return self._xCol.value
    @xColumn.setter
    def xColumn(self, value): self._xCol.value = value

    @property
    def yColumn(self): return self._yCol.value
    @yColumn.setter
    def yColumn(self, value): self._yCol.value = value

    @property
    def zColumn(self): return self._zCol.value
    @zColumn.setter
    def zColumn(self, value): self._zCol.value = value

    @property
    def loadButton(self): return self._loadButton

    @property
    def xField(self): return self._xCol

    @property
    def yField(self): return self._yCol

    @property
    def zField(self): return self._zCol

    def __iter__(self):
        if self._filename.value != None and self._filename.value != '':

            csvfile = open(self._filename.value, 'U')
            self._spamreader = csv.reader(csvfile, delimiter=self._separator.value)
            for i in range(int(self._startingrow.value)): next(self._spamreader, None)  # skip the headers
            self._cols = [self.frameColumn]
            if self.xField.visible:
                self._cols.append(self.xColumn)
            if self.yField.visible:
                self._cols.append(self.yColumn)
            if self.zField.visible:
                self._cols.append(self.zColumn)
        else:
            self._spamreader = None
        return self

    # For compatibility with python 3
    def __next__(self): return self.next()

    def next(self):
        if self._spamreader != None:
            row = next(self._spamreader)
            return [row[int(col)] for col in self._cols]
        else:
            raise StopIteration()

    def __refreshPreview(self):
        if self._filename.value != None and self._filename.value != '':
            with open(self._filename.value, 'U') as csvfile:
                spamreader = csv.reader(csvfile, delimiter=self._separator.value)
                for i in range(int(self._startingrow.value)): next(spamreader, None)  # skip the headers
                self._filePreview.value = []
                self._filePreview.horizontalHeaders = ["0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "10", "11", "12", "13", "14", "15", "16", ]
                for i, row in enumerate(spamreader):
                    self._filePreview += row
                    if i >= 10:
                        break

    def load(self): 
        self.__refreshPreview()
        self._loadButton.value()

##################################################################################################################
##################################################################################################################
##################################################################################################################

# Execute the application
if __name__ == "__main__":
    import pyforms
    pyforms.start_app(CsvParserDialog, geometry=(0, 0, 600, 400))



from pyforms.basewidget import BaseWidget
from pyforms_gui.controls.control_button import ControlButton
from pyforms_gui.controls.control_text import ControlText
from pyforms_gui.controls.control_number import ControlNumber
from pyforms_gui.controls.control_checkbox import ControlCheckBox

from confapp import conf
from AnyQt.QtGui import QColor
from AnyQt.QtWidgets import QColorDialog

class EventWindow(BaseWidget):

    def __init__(self, parent_win, event):
        BaseWidget.__init__(self, 'Edit frame', parent_win=parent_win)

        self.event = event
        self._widget = parent_win

        self.set_margin(5)

        self._applybtn = ControlButton('Apply', default=self.__apply_evt)
        self._label = ControlText('Label', default=event.title)
        self._begin = ControlNumber(
            'Begin',
            default=event.begin,
            minimum=0,
            maximum=100000000000000,
            changed_event=self.__begin_changed_event
        )
        self._end   = ControlNumber(
            'End',
            default=event.end,
            minimum=0,
            maximum=100000000000000,
            changed_event=self.__end_changed_event
        )
        self._color = ControlButton(str(event.color.name()), default=self.__pick_color_evt)
        self._lock  = ControlCheckBox('Locked', default=event.lock)

        self.formset = [
            '_label',
            ('_begin', '_end'),
            ('_color', '_lock'),
            '_applybtn'
        ]

    def update_form(self, event):
        self.event = event
        self._label.value = event.title
        self._end.value = int(event.end)
        self._begin.value = int(event.begin)
        self._lock.value = event.lock
        self._color.label = str(event.color.name())

    def __apply_evt(self):
        self.event.title = self._label.value
        self.event.end = self._end.value
        self.event.begin = self._begin.value
        self.event.lock = self._lock.value
        self.event.color = self._color.label
        self._widget.repaint()
        self.close()

    def __pick_color_evt(self):
        color = QColorDialog.getColor(
            initial = QColor(self._color.label),
            parent = self._widget,
            options = conf.PYFORMS_COLORDIALOGS_OPTIONS
        )
        if color:
            self._color.label = color.name()

    def __begin_changed_event(self):
        if not hasattr(self, '_updating') and self._begin.value >= self._end.value:
            self._updating = True
            self._begin.value = self._end.value - 1
            del self._updating

    def __end_changed_event(self):
        if not hasattr(self, '_updating') and self._end.value <= self._begin.value:
            self._updating = True
            self._end.value = self._begin.value + 1
            del self._updating




# !/usr/bin/python
# -*- coding: utf-8 -*-

import logging

from AnyQt import _api
from confapp import conf

from pyforms_gui.controls.control_base import ControlBase
import pyforms_gui.utils.tools as tools

from AnyQt 			 import QtCore, uic
from AnyQt.QtWidgets import QApplication, QWidget, QVBoxLayout, QHBoxLayout, QSpinBox
from AnyQt.QtGui 	 import QFontMetrics, QColor, QIcon, QFont

if _api.USED_API == _api.QT_API_PYQT5:
    from PyQt5.Qsci import QsciScintilla, QsciLexerPython, QsciAPIs
elif _api.USED_API == _api.QT_API_PYQT4:
    from PyQt4.Qsci import QsciScintilla, QsciLexerPython, QsciAPIs


logger = logging.getLogger(__name__)


class ControlCodeEditor(ControlBase):
	"""
	Control that offers a code editor with pretty-print and line numbers and a save button
	"""

	ARROW_MARKER_NUM = 8

	def __init__(self, *args, **kwargs):
		"""
		
		:param label: 
		:param default: 
		:param helptext: 
		"""
		self._read_only = kwargs.get('readonly', False)
		self._changed_func = None
		super(ControlCodeEditor, self).__init__(*args, **kwargs)

		self.discard_event = kwargs.get('discard_event', self.discard_event)
		
	def init_form(self):
		"""
		
		"""

		control_path = tools.getFileInSameDirectory(__file__, "code_editor.ui")
		self._form = uic.loadUi(control_path)

		self._code_editor = self._form.code_editor
		self._save_button = self._form.save_button
		self._discard_button = self._form.discard_button

		self._save_button.clicked[bool].connect(self.on_save_changes)
		self._discard_button.clicked[bool].connect(self.on_discard_changes)

		if self._read_only:
			self._code_editor.setReadOnly(True)
			self._save_button.setVisible(False)
			self._discard_button.setVisible(False)

		self.form.font_size.addItem('9')
		self.form.font_size.addItem('10')
		self.form.font_size.addItem('11')
		self.form.font_size.addItem('12')
		self.form.font_size.addItem('14')
		self.form.font_size.addItem('18')
		self.form.font_size.addItem('24')

		# Set the default font size
		index = self.form.font_size.findText(conf.PYFORMS_CONTROL_CODE_EDITOR_DEFAULT_FONT_SIZE)
		self.form.font_size.setCurrentIndex(index)

		self.form.font_size.currentIndexChanged.connect(self.__font_size_index_changed)

		self.form.save_button.setIcon(QIcon(conf.PYFORMS_ICON_CODEEDITOR_SAVE))
		self.form.discard_button.setIcon(QIcon(conf.PYFORMS_ICON_CODEEDITOR_DISCARD))

		self.lexer = QsciLexerPython

		self._code_editor.keyPressEvent = self._key_pressed
		self._changed_func = None

		self.value = self._value
		super(ControlCodeEditor, self).init_form()

	def __font_size_index_changed(self, index):
		item = self.form.font_size.currentText()
		if len(item) >= 1:
			self._load_code_editor_settings()

	def _load_code_editor_settings(self):
		"""
		Load settings on the code editor like, font style, margins, scroll, etc.
		Based on the example from http://eli.thegreenplace.net/2011/04/01/sample-using-qscintilla-with-pyqt/
		"""

		item = self.form.font_size.currentText()
		size = int(item)

		# Set the default font
		font = QFont()
		font.setFamily('Courier New')
		font.setFixedPitch(True)
		font.setPointSize(size)
		self._code_editor.setFont(font)
		self._code_editor.setMarginsFont(font)

		# Margin 0 is used for line numbers
		fontmetrics = QFontMetrics(font)
		self._code_editor.setMarginsFont(font)
		self._code_editor.setMarginWidth(0, fontmetrics.width("00000") + 6)
		self._code_editor.setMarginLineNumbers(0, True)
		self._code_editor.setMarginsBackgroundColor(QColor("#cccccc"))

		# Clickable margin 1 for showing markers
		self._code_editor.setMarginSensitivity(1, True)
		self._code_editor.marginClicked.connect(self.on_margin_clicked)
		self._code_editor.markerDefine(QsciScintilla.RightArrow, self.ARROW_MARKER_NUM)
		self._code_editor.setMarkerBackgroundColor(QColor("#ee1111"), self.ARROW_MARKER_NUM)

		# Detect changes to text
		self._code_editor.modificationChanged.connect(self.on_modification_changed)

		# Brace matching: enable for a brace immediately before or after the current position
		self._code_editor.setBraceMatching(QsciScintilla.SloppyBraceMatch)

		# Current line visible with special background color
		self._code_editor.setCaretLineVisible(True)
		self._code_editor.setCaretLineBackgroundColor(QColor("#ffe4e4"))

		# Set Python lexer
		# Set style for Python comments (style number 1) to a fixed-width Courier.
		lexer = self.lexer()
		lexer.setDefaultFont(font)
		self._code_editor.setLexer(lexer)
		self._code_editor.setIndentationWidth(4)
		# self._code_editor.SendScintilla(QsciScintilla.SCI_STYLESETFONT, 1, 'Courier')
		self._code_editor.SendScintilla(QsciScintilla.SCI_STYLESETFONT, 1)

		# Don't want to see the horizontal scrollbar at all
		# Use raw message to Scintilla here (all messages are documented here: http://www.scintilla.org/ScintillaDoc.html)
		self._code_editor.SendScintilla(QsciScintilla.SCI_SETHSCROLLBAR, 0)
		
		self._lexer_obj = lexer
		self.qsci_api = QsciAPIs(self._lexer_obj)
		## Add autocompletion strings
		self.qsci_api.add("aLongString")
		self.qsci_api.add("aLongerString")
		self.qsci_api.add("aDifferentString")
		self.qsci_api.add("sOmethingElse")
		## Compile the api for use in the lexer
		self.qsci_api.prepare()
		self._code_editor.setAutoCompletionThreshold(1)
		self._code_editor.setAutoCompletionSource(QsciScintilla.AcsAll)

	# not too small
	# self._code_editor.setMinimumSize(600, 450)

	###################################################################
	############ Events ###############################################
	###################################################################

	def on_margin_clicked(self, nmargin, nline, modifiers):  # pylint: disable=unused-argument
		"""
		On margin clicked, toggle marker for the line the margin was clicked on
		:param nmargin:
		:type nmargin:
		:param nline:
		:type nline:
		:param modifiers:
		:type modifiers:
		"""
		if self._code_editor.markersAtLine(nline) != 0:
			self._code_editor.markerDelete(nline, self.ARROW_MARKER_NUM)
		else:
			self._code_editor.markerAdd(nline, self.ARROW_MARKER_NUM)

	def on_modification_changed(self):
		"""
		On modification change, re-enable save button
		"""
		if self._changed_func: 
			self._save_button.setEnabled(True)
			self._discard_button.setEnabled(True)

	def on_save_changes(self):
		"""
		On button save clicked, save changes made on the code editor to file
		"""
		if self.changed_event():
			self._code_editor.setModified(False)
			self._save_button.setEnabled(False)
			self._discard_button.setEnabled(False)

	def on_discard_changes(self):

		if self.discard_event():
			self._code_editor.setModified(False)
			self._save_button.setEnabled(False)
			self._discard_button.setEnabled(False)

	def discard_event(self):

		return True


	def _key_pressed(self, event):
		"""
		Handle KeyPressed event
		We only care about CTRL-S in order to save changes
		:param event: key event
		"""
		QsciScintilla.keyPressEvent(self._code_editor, event)
		if event.key() in [QtCore.Qt.Key_S, QtCore.Qt.Key_Save]:
			modifiers = QApplication.keyboardModifiers()
			if modifiers == QtCore.Qt.ControlModifier and self.is_modified:
				logger.debug("Saving...")
				self.on_save_changes()

		self.key_pressed_event(event)

	def key_pressed_event(self, event):
		"""
		Override KeyPressed event as you like
		:param event: key event
		"""
		pass

	@property
	def is_modified(self):
		return self._code_editor.isModified()

	###################################################################
	############ Properties ###########################################
	###################################################################

	@property
	def lexer(self):
		return self._lexer

	@lexer.setter
	def lexer(self, value):
		self._lexer = value
		self._load_code_editor_settings()

	@property
	def value(self):
		return self._code_editor.text()

	@value.setter
	def value(self, value):
		if value is not None:
			self._code_editor.setText(str(value))
			self._code_editor.setModified(False)
			self._save_button.setEnabled(False)
			self._discard_button.setEnabled(False)

	@property
	def changed_event(self):
		return self._changed_func if self._changed_func else (lambda: 0)

	@changed_event.setter
	def changed_event(self, value):
		self._changed_func = value



#!/usr/bin/python
# -*- coding: utf-8 -*-

from confapp import conf

from AnyQt           import QtCore, _api
from AnyQt.QtWidgets import QWidget, QVBoxLayout, QHBoxLayout, QSpinBox
from AnyQt.QtGui     import QFont, QColor, QCursor, QPainter
from AnyQt.QtWidgets import QLabel, QSizePolicy

from pyforms_gui.controls.control_base import ControlBase


class GaugeWidgetVertical(QWidget):
    def __init__(self, *args, **kwargs):
        QWidget.__init__(self, *args, **kwargs)

        self.setMouseTracking(True)
        self.setMinimumHeight(30)
        self.setMinimumWidth(30)
        self.setMaximumWidth(50)

        self.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Fixed)

        self._step = 0
        self._lower = 0
        self._higher = 100
        self._minVal = 0
        self._maxVal = 75
        self._lastMouseY = None
        self._moving = False
        self._resizingBottom = False
        self._resizingTop = False
        self._scale = 1.0
        self._use_float = False

    def changed_event(self):
        pass

    def paintEvent(self, e):
        # call the base implementation to paint normal interface
        super(GaugeWidgetVertical, self).paintEvent(e)
        draw = QPainter()
        draw.begin(self)

        window_with = self.width() - 1
        diff = (self._higher - self._lower) * self.scale
        try:
            self._step = float(self.height()) / float(diff)
        except ZeroDivisionError:
            self._step = 0
        y_start = self.height() - (self._minVal - self._lower) * self._step * self.scale
        y_end = self.height() - (self._maxVal - self._lower) * self._step * self.scale

        draw.setOpacity(1.0)
        draw.setBrush(QtCore.Qt.NoBrush)
        draw.setPen(QColor(200, 200, 255))

        b = int(self.height() / 5)
        e = int(self.height() - (self.height() / 5) + 1)
        s = int(self.height() / 5)

        for i in range( b, e, s ):
            draw.drawLine(0, i, window_with, i)

        draw.setBrush(QColor(33, 133, 208))
        draw.setPen(QColor(33, 133, 208))
        draw.setOpacity(0.7)
        draw.drawRect(0, y_start, window_with, y_end - y_start)

        draw.setFont(QFont('Decorative', 8))
        draw.setPen(QColor(30, 30, 30))
        draw.drawText(3, self.height() - 3, str(self._lower) if self._use_float else str(int(round(self._lower))))
        draw.drawText(3, 10, str(self._higher) if self._use_float else str(int(round(self._higher))))

        draw.drawText(3, y_start + 11, str(self._minVal) if self._use_float else str(int(round(self._minVal))))
        draw.drawText(3, y_end - 4, str(self._maxVal) if self._use_float else str(int(round(self._maxVal))))

        draw.end()

    def mousePressEvent(self, event):
        self._moving = False
        self._resizingBottom = False
        self._resizingTop = False

        y_start = self.height() - (self._minVal - self._lower) * self._step * self.scale
        y_end = self.height() - (self._maxVal - self._lower) * self._step * self.scale

        if event.buttons() == QtCore.Qt.LeftButton:
            if y_start > event.y() > y_end:
                self._moving = True
            elif event.y() < y_end:
                self._resizingTop = True
            elif y_start < event.y():
                self._resizingBottom = True

    def mouseReleaseEvent(self, event):
        self._moving = False
        self._resizingBottom = False
        self._resizingTop = False

    def mouseMoveEvent(self, event):
        super(GaugeWidgetVertical, self).mouseMoveEvent(event)

        y_start = self.height() - (self._minVal - self._lower) * self._step * self.scale
        y_end = self.height() - (self._maxVal - self._lower) * self._step * self.scale

        if y_start < event.y() or event.y() < y_end:
            self.setCursor(QCursor(QtCore.Qt.SizeVerCursor))
        else:
            self.setCursor(QCursor(QtCore.Qt.ArrowCursor))

        if event.buttons() == QtCore.Qt.LeftButton:
            if self._lastMouseY != None:
                diff = self._lastMouseY - event.y()
                if diff != 0:
                    if self._moving:
                        new_min = self._minVal * self.scale + diff * 1 / self._step
                        new_max = self._maxVal * self.scale + diff * 1 / self._step
                        if new_min < new_max:
                            self._minVal = new_min / self.scale
                            self._maxVal = new_max / self.scale
                    elif self._resizingTop:
                        new_max = self._maxVal * self.scale + diff * 1 / self._step
                        if self._minVal < (new_max / self.scale): self._maxVal = new_max / self.scale
                    elif self._resizingBottom:
                        new_min = self._minVal * self.scale + diff * 1 / self._step
                        if (new_min / self.scale) < self._maxVal: self._minVal = new_min / self.scale

                    self.repaint()

                    self.changed_event()

            self._lastMouseY = event.y()

    def mouseReleaseEvent(self, event):
        self._lastMouseY = None

    @property
    def scale(self):
        return self._scale

    @scale.setter
    def scale(self, value):
        self._scale = value


class GaugeWidgetHorizontal(GaugeWidgetVertical):
    def __init__(self, *args, **kwargs):
        super(GaugeWidgetHorizontal, self).__init__(*args, **kwargs)

        self.setMaximumWidth(1000000)
        self.setMinimumHeight(20)
        self.setMaximumHeight(20)

    def paintEvent(self, e):
        # call the base implementation to paint normal interface
        QWidget.paintEvent(self, e);
        draw = QPainter();
        draw.begin(self)

        h = self.height() - 1
        diff = (self._higher - self._lower) * self.scale

        try:
            self._step = float(self.width()) / float(diff)
        except ZeroDivisionError:
            self._step = 0
        x_start = (self._minVal - self._lower) * self._step * self.scale
        x_end = (self._maxVal - self._lower) * self._step * self.scale

        draw.setOpacity(1.0)
        draw.setBrush(QtCore.Qt.NoBrush)
        draw.setPen(QColor(200, 200, 255))

        for i in range( int(self.width()/5), int(self.width()-self.width()/5) + 1, int(self.width()/5) ): 
            draw.drawLine(i, 0, i, h)

        draw.setBrush(QColor(238, 238, 238))
        draw.setPen(QColor(238, 238, 238))
        draw.drawRoundedRect(0, 2, self.width(), h - 4, 3, 3)

        draw.setBrush(QColor(33, 133, 208))
        draw.setPen(QColor(33, 133, 208))
        draw.drawRoundedRect(int(round(x_start)), 2, int(round(x_end - x_start)), h - 4, 3, 3)
        # draw.setOpacity(1.0)
        draw.setFont(QFont('Decorative', 8))
        draw.setPen(QColor(80, 80, 80))

        str(self._maxVal) if self._use_float else str(int(round(self._maxVal)))

        boundtext = draw.boundingRect(QtCore.QRectF(),
                                      str(self._higher) if self._use_float else str(int(round(self._higher))))
        draw.drawText(self.width() - boundtext.width(), 14,
                      str(self._higher) if self._use_float else str(int(round(self._higher))))
        draw.drawText(0, 14, str(self._lower) if self._use_float else str(int(round(self._lower))))

        draw.setPen(QColor(255, 255, 255))
        boundtext = draw.boundingRect(QtCore.QRectF(),
                                      str(self._minVal) if self._use_float else str(int(round(self._minVal))))
        draw.drawText(x_start + 2, 14, str(self._minVal) if self._use_float else str(int(round(self._minVal))))
        boundtext = draw.boundingRect(QtCore.QRectF(),
                                      str(self._maxVal) if self._use_float else str(int(round(self._maxVal))))
        draw.drawText(x_end - boundtext.width(), 14,
                      str(self._maxVal) if self._use_float else str(int(round(self._maxVal))))

        draw.end()

    def mousePressEvent(self, event):
        self._moving = False
        self._resizingLeft = False
        self._resizingRight = False

        x_start = (self._minVal - self._lower) * self._step * self.scale
        x_end = (self._maxVal - self._lower) * self._step * self.scale

        if event.buttons() == QtCore.Qt.LeftButton:
            if x_start < event.x() < x_end:
                self._moving = True
            elif event.x() > x_end:
                self._resizingLeft = True
            elif x_start > event.x():
                self._resizingRight = True

    def mouseMoveEvent(self, event):
        super(GaugeWidgetVertical, self).mouseMoveEvent(event)

        x_start = (self._minVal - self._lower) * self._step * self.scale
        x_end = (self._maxVal - self._lower) * self._step * self.scale

        # set the cursors
        if x_start >= event.x() or event.x() >= x_end:
            self.setCursor(QCursor(QtCore.Qt.SizeHorCursor))
        else:
            self.setCursor(QCursor(QtCore.Qt.ArrowCursor))

        if event.buttons() == QtCore.Qt.LeftButton:
            if self._lastMouseY != None:
                diff = self._lastMouseY - event.x()
                if diff != 0:
                    if self._moving:
                        new_min = self._minVal * self.scale - diff * 1 / self._step
                        new_max = self._maxVal * self.scale - diff * 1 / self._step
                        if new_min < new_max:
                            self._minVal = new_min / self.scale
                            self._maxVal = new_max / self.scale
                    elif self._resizingLeft:
                        new_max = self._maxVal * self.scale - diff * 1 / self._step
                        if self._minVal < (new_max / self.scale): self._maxVal = new_max / self.scale
                    elif self._resizingRight:
                        new_min = self._minVal * self.scale - diff * 1 / self._step
                        if (new_min / self.scale) < self._maxVal: self._minVal = new_min / self.scale

                    self.repaint()
                    self.changed_event()

            self._lastMouseY = event.x()


class ControlBoundingSlider(ControlBase):
    """
    .. image:: https://raw.githubusercontent.com/UmSenhorQualquer/pyforms/master/tutorials/Controls4Docs/ControlBoundingSlider.png  

    
    """

    def __init__(self, *args, **kwargs):
        """
        
        :param tupple default: The default value is a list containing in the first element the lower value and in the second element the upper value. Default = [20,40].
        :param bool horizontal: Flag indicating if the Bounding slider should be draw horizontally or vertically. Default = True.
        :param bool show_spinboxes: Show or hide the spinboxes. Default = True
        :param float minimum: Defines the minimum value that can be selected.
        :param float maximum: Defines the maximum value that can be selected.  
        :param bool convert_2_int: Flag to define if the control should return floats or integers.
        
        """
        self._horizontal     = kwargs.get('horizontal', True)
        self._show_spinboxes = kwargs.get('show_spinboxes', True)
        ControlBase.__init__(self,  *args, **kwargs)

        self.min = kwargs.get('min', kwargs.get('minimum', 0))
        self.max = kwargs.get('max', kwargs.get('maximum', 100))
        self.value = kwargs.get('default', [10,20])
        self.convert_2_int = kwargs.get('convert_2_int', False)
        self.__update()

    def init_form(self):
        self._boundingbox = GaugeWidgetHorizontal() if self._horizontal else GaugeWidgetVertical()
        self._boundingbox.changed_event = self.__update

        if self._show_spinboxes:
            self._form = hwidget = QWidget()
            if self._horizontal:
                hlayout = QHBoxLayout()
            else:
                hlayout = QVBoxLayout()
            

            if _api.USED_API == _api.QT_API_PYQT5:
                hlayout.setContentsMargins(0,0,0,0)
            elif _api.USED_API == _api.QT_API_PYQT4:
                hlayout.setMargin(0)

            if self._label is not None:
                self._controllabel = QLabel(self.form)
                hlayout.addWidget(self._controllabel)
                self._controllabel.setAccessibleName('ControlBoundingSlider-label')
                self.label = self._label
            else:
                self._controllabel = None


            hwidget.setLayout(hlayout)
            self._min_spinbox = QSpinBox()
            self._min_spinbox.valueChanged.connect(self.__min_spinbox_changed)
            self._min_spinbox.setMaximumWidth(95)

            self._max_spinbox = QSpinBox()
            self._max_spinbox.valueChanged.connect(self.__max_spinbox_changed)
            self._max_spinbox.setMaximumWidth(95)

            if self._horizontal:
                hlayout.addWidget(self._min_spinbox)
            else:
                hlayout.addWidget(self._max_spinbox)
            hlayout.addWidget(self._boundingbox)
            if self._horizontal:
                hlayout.addWidget(self._max_spinbox)
            else:
                hlayout.addWidget(self._min_spinbox)

        else:
            self._form = self._boundingbox

        super(ControlBoundingSlider, self).init_form()

    def __max_spinbox_changed(self, value):
        if value < self._boundingbox._minVal: return
        if hasattr(self, '_is_updating_spinboxes'): return
        self.scale = self.__find_scale_factor(value)
        self._boundingbox._maxVal = value
        self._boundingbox.repaint()
        self.changed_event()

    def __min_spinbox_changed(self, value):
        if value > self._boundingbox._maxVal: return
        if hasattr(self, '_is_updating_spinboxes'): return
        self.scale = self.__find_scale_factor(value)
        self._boundingbox._minVal = value
        self._boundingbox.repaint()
        self.changed_event()

    def __update(self):
        l, h = self._boundingbox._minVal, self._boundingbox._maxVal
        self._is_updating_spinboxes = True
        if self._show_spinboxes:
            self._min_spinbox.setValue(l)
            self._max_spinbox.setValue(h)
        del self._is_updating_spinboxes
        self.changed_event()

    def changed_event(self):
        pass

    def __find_scale_factor(self, value):
        scale = 1.0
        new_value = value
        while abs(new_value) < 0.0:
            scale *= 10.0
            new_value = value * scale
        return scale

    def load_form(self, data, path=None):
        """
        Load a value from the dict variable
        @param data: dictionary with the value of the Control
        """

        self.convert_2_int = data.get('convert-int', self.convert_2_int)
        self.scale = data.get('scale', self.scale)
        self.max = data.get('max', self.max)
        self.min = data.get('min', self.min)
        self.value = data.get('value', self.value)
        

    def save_form(self, data, path=None):
        """
        Save a value to dict variable
        @param data: dictionary with to where the value of the Control will be added
        """
        data['value']       = self.value
        data['max']         = self.max
        data['min']         = self.min
        data['scale']       = self.scale
        data['convert-int'] = self.convert_2_int
        return data

    ##########################################################################
    ############ Properties ##################################################
    ##########################################################################

    @property
    def label(self):
        return self._controllabel.getText()
    @label.setter
    def label(self, value):
        self._controllabel.setText(value)

    @property
    def value(self):
        """
        Sets and gets the control value. It should be a list or tuple of 2 values.
        """
        return self._boundingbox._minVal, self._boundingbox._maxVal

    @value.setter
    def value(self, value):
        self.scale = self.__find_scale_factor(value[0])
        self._boundingbox._minVal, self._boundingbox._maxVal = value[0], value[1]
        if hasattr(self, '_min_spinbox'): self._min_spinbox.setValue(value[0])
        if hasattr(self, '_max_spinbox'): self._max_spinbox.setValue(value[1])
        ControlBase.value.fset(self, value)
        self._boundingbox.repaint()

    @property
    def min(self):
        """
        Sets and gets the minimum value possible.
        """
        return self._boundingbox._lower

    @min.setter
    def min(self, value):
        self._boundingbox._lower = value
        self._boundingbox.repaint()
        if hasattr(self, '_min_spinbox'): self._min_spinbox.setMinimum(value)
        if hasattr(self, '_max_spinbox'): self._max_spinbox.setMinimum(value)

    @property
    def max(self):
        """
        Sets and gets the maximum value possible.
        """
        return self._boundingbox._higher

    @max.setter
    def max(self, value):
        self._boundingbox._higher = value
        self._boundingbox.repaint()
        if hasattr(self, '_min_spinbox'): self._min_spinbox.setMaximum(value)
        if hasattr(self, '_max_spinbox'): self._max_spinbox.setMaximum(value)

    @property
    def scale(self):
        """
        Sets and gets the scale value.
        """
        return self._boundingbox.scale

    @scale.setter
    def scale(self, value):
        self._boundingbox.scale = value

    @property
    def convert_2_int(self):
        """
        Flag to define if the control should return floats or integers.
        """
        return not self._boundingbox._use_float

    @convert_2_int.setter
    def convert_2_int(self, value):
        self._boundingbox._use_float = not value



import sys, os
sys.path.append(os.getcwd())


def start():
    import logging, locale, coloredlogs
    logger = logging.getLogger(__name__)
    logging.basicConfig(filename='idtrackerai-app.log', filemode='w',
                        format='%(name)s - %(levelname)s - %(message)s')
    coloredlogs.install(
        level='DEBUG',
        fmt='[%(levelname)-8s] %(name)-40s %(message)s',
        #stream=open("idtrackerai-app.log", 'w')
    )
    coloredlogs.install(
        level='DEBUG',
        fmt='[%(levelname)-8s] %(name)-40s %(message)s',
        stream=open("idtrackerai-app.log", 'w')
    )

    #logging.getLogger('').addHandler(open("idtrackerai-app.log", 'w'))

    from pyforms import start_app
    from confapp import conf

    logging.getLogger('PyQt5').setLevel(logging.INFO)
    logging.getLogger('matplotlib').setLevel(logging.INFO)

    if conf.PYFORMS_MODE=='GUI':
        from .win_idtrackerai  import IdTrackerAiGUI as App
    else:
        from .base_idtrackerai  import BaseIdTrackerAi as App


    try:
        start_app( App, geometry=(100, 100, 800, 600) )
    except SystemExit:
        pass
    except Exception as e:
        logger.info(e, exc_info=True)
        import sys
        import traceback
        ex_type, ex, tb = sys.exc_info()
        traceback.print_exception(ex_type, ex, tb)


#Execute the application
if __name__ == "__main__":
    start()



import os, subprocess, cv2, numpy as np, sys
import glob
from confapp import conf

from PyQt5.QtWidgets import QApplication
from AnyQt.QtWidgets import QMessageBox

from idtrackerai.utils.segmentation_utils import segment_frame, blob_extractor
from idtrackerai.postprocessing.individual_videos import generate_individual_videos
from idtrackerai.postprocessing.trajectories_to_video import generate_trajectories_video

from pyforms.controls import ControlButton
from pyforms.controls import ControlPlayer
from pyforms.controls import ControlCheckBox

from .base_idtrackerai import BaseIdTrackerAi
from .gui.grapharea_win import GraphAreaWin
from .gui.range_win import RangeWin


class IdTrackerAiGUI(BaseIdTrackerAi):

    INDIV_VIDEO_BTN_LABEL = 'Generate individual videos'
    GEN_INDIV_VIDEO_BTN_LABEL = 'Generating individual videos...'
    TRAJ_VIDEO_BTN_LABEL = 'Generate video with trajectories'
    GEN_TRAJ_VIDEO_BTN_LABEL = 'Generate video with trajectories...'
    FINAL_MESSAGE_DEFAULT = "idtracker.ai terminated."

    def __init__(self, *args, **kwargs):

        self._player            = ControlPlayer('Player', enabled=False, multiple_files=True)
        self._togglegraph       = ControlCheckBox('Segmented blobs info', changed_event=self.__toggle_graph_evt, enabled=False)
        self._pre_processing    = ControlButton('Track video', default=self.track_video, enabled=False)
        self._savebtn           = ControlButton('Save parameters', default=self.save_window, enabled=False)

        self._polybtn           = ControlButton('Polygon', checkable=True, enabled=False, default=self.polybtn_click_evt)
        self._rectbtn           = ControlButton('Rectangle', checkable=True, enabled=False)
        self._circlebtn         = ControlButton('Ellipse', checkable=True, enabled=False, default=self.circlebtn_click_evt)

        self._addrange          = ControlButton('Add interval', default=self.__rangelst_add_evt, visible=False)

        self._indiv_videos      = ControlButton(self.INDIV_VIDEO_BTN_LABEL, default=self.__generate_individual_videos_evt, enabled=False)
        self._traj_video        = ControlButton(self.TRAJ_VIDEO_BTN_LABEL, default=self.__generate_trajectories_video_evt, enabled=False)
        self._validation        = ControlButton('Validate trajectories', default=self.__open_videoannotator_evt, enabled=False)

        self._final_message     = self.FINAL_MESSAGE_DEFAULT

        self._graph = GraphAreaWin(parent_win=self)

        super().__init__(*args, **kwargs)

        self.set_margin(10)

        self.formset = [
            ('_video', '_session', '_savebtn'),
            '_player',
            '=',
            ('_nblobs', '_togglegraph', '_chcksegm', ' '),
            ('_intensity', '_bgsub'),
            ('_area', '_resreduct'),
            ('_range', '_rangelst', '_addrange', '_multiple_range'),
            ('_applyroi', ' '),
            ('_rectbtn', '_polybtn', '_circlebtn', ' '),
            '_roi',
            ('_no_ids', '_pre_processing', '_progress', '_validation'),
            ('_indiv_videos', '_traj_video', ' ')
        ]

        self._graph.on_draw = self.__graph_on_draw_evt
        self._applyroi.changed_event = self.__apply_roi_changed_evt
        self._session.changed_event = self.__session_changed_evt
        self._player.drag_event = self.on_player_drag_in_video_window
        self._player.end_drag_event = self.on_player_end_drag_in_video_window
        self._player.click_event = self.on_player_click_in_video_window
        self._video.changed_event = self.__video_changed_evt
        self._video_path.changed_event = self.__video_changed_evt
        self._intensity.changed_event = self._player.refresh
        self._player.double_click_event = self.on_player_double_click_in_video_window
        self._player.process_frame_event = self.process_frame_evt
        self._multiple_range.changed_event = self.__multiple_range_changed_evt
        self._resreduct.changed_event = self.__resreduct_changed_evt

        self.setMinimumHeight(900)
        self._player.setMinimumHeight(300)
        self._togglegraph.form.setMaximumWidth(180)
        self._roi.setMaximumHeight(100)
        self._session.form.setMaximumWidth(250)
        self._multiple_range.form.setMaximumWidth(130)

        self.__apply_roi_changed_evt()
        self.__session_changed_evt()

    def save_form(self, data={}, path=None):
        data['open-multiple-files'] = self.open_multiple_files
        return super().save_form(data, path)

    def track_video(self):
        super().track_video()
        self.info(self._final_message, 'Ended')

    def set_controls_enabled(self, status):
        super().set_controls_enabled(status)
        self._validation.enabled = status
        self._indiv_videos.enabled = status
        self._traj_video.enabled = status
        self._pre_processing.enabled = status
        self._savebtn.enabled = status
        self._player.enabled = status
        self._polybtn.enabled = status
        self._rectbtn.enabled = status
        self._circlebtn.enabled = status
        self._togglegraph.enabled = status
        self._addrange.enabled = status


    def process_frame_evt(self, frame):
        """
        Function called before an image is shown in the player.
        It does the pre-visualization segmentation and ROIs selection.
        """
        min_thresh, max_thresh = self._intensity.value
        min_area,   max_area   = self._area.value

        original_size = frame.shape[1], frame.shape[0]
        reduction = self._resreduct.value
        frame = cv2.resize(frame, None, fx=reduction, fy=reduction, interpolation=cv2.INTER_AREA)


        # Convert the frame to black & white
        gray = cv2.cvtColor(frame, cv2.COLOR_RGB2GRAY) if len(frame.shape)>2 else frame
        mask = self.create_mask(*gray.shape)
        av_intensity = np.float32(np.mean(np.ma.array(gray, mask=mask==0)))
        av_frame     = gray / av_intensity


        bin_frame    = segment_frame( av_frame, min_thresh, max_thresh, self._background_img, mask, self._bgsub.value)
        boxes, mini_frames, _, areas, _, good_cnt, _ = blob_extractor(bin_frame.copy(), frame, int(min_area), int(max_area))
        self._detected_areas = areas
        #if self._nblobs.value<len(areas):
        #    self._nblobs.value = len(areas)

        cv2.drawContours(frame, good_cnt, -1, color=(0,0,255), thickness=-1)

        if conf.PYFORMS_MODE == 'GUI' and self._togglegraph.value:
            self._graph.draw()

        # The resize to the original size is required because of the draw of the ROI.
        frame = cv2.resize(frame, original_size, interpolation=cv2.INTER_AREA)

        self.draw_rois(frame)
        return frame

    def __resreduct_changed_evt(self):
        self._player.refresh()

    def __rangelst_add_evt(self):
        win = RangeWin(
            parent_win=self,
            maximum=self._player.max,
            begin=self._player.video_index,
            control_list=self._rangelst
        )
        win.show()

    def __multiple_range_changed_evt(self):
        if self._multiple_range.value:
            self._range.hide()
            self._addrange.show()
            self._rangelst.show()
        else:
            self._range.show()
            self._addrange.hide()
            self._rangelst.hide()

    def __video_changed_evt(self):
        """
        Function called when the video file is selected.
        Ask to the player to load the video file.
        """
        if self.video_path:
            self._player.value = self.video_path
            if self._player.value:
                self._range.max   = self._player.max
                self._range.value = [0, self._player.max]
                self.set_controls_enabled(True)
                self._player.forward_one_frame()
            else:
                self.set_controls_enabled(False)

        else:
            self.set_controls_enabled(False)

    def bgsub_changed_evt(self):
        super().bgsub_changed_evt()
        self._player.refresh()

    def __toggle_graph_evt(self):
        if self._togglegraph.value:
            self._graph.show()
        else:
            self._graph.hide()

    def __graph_on_draw_evt(self, figure):
        areas = self._detected_areas

        if not areas: return

        axes = figure.add_subplot(111)
        axes.clear()
        axes.bar(range(1,len(areas)+1), areas, 0.5)
        if len(areas) > 0:
            min_area = np.min(areas)
            axes.axhline(min_area, color = 'k', linewidth = .3)
            axes.set_title( str(len(areas)) + " blobs detected. Minimum area: " + str(min_area) )
            axes.set_xlabel('Blob index')
            axes.set_ylabel('Area in pixels')

    def __apply_roi_changed_evt(self):
        """
        Hide and show the controls to setup the background
        """
        if self._applyroi.value:
            self._roi.show()
            self._polybtn.show()
            self._rectbtn.show()
            self._circlebtn.show()
        else:
            self._roi.hide()
            self._polybtn.hide()
            self._rectbtn.hide()
            self._circlebtn.hide()

    def __session_changed_evt(self):
        video_folder   = os.path.dirname(self.video_path)
        session_folder = "session_{0}".format(self._session.value)
        session_path   = os.path.join(video_folder, session_folder)
        blobs_path_1   = os.path.join(session_path, 'preprocessing', 'blobs_collection_no_gaps.npy')
        blobs_path_2   = os.path.join(session_path, 'preprocessing', 'blobs_collection.npy')
        vidobj_path    = os.path.join(session_path, 'video_object.npy')
        trajs_wo_path  = os.path.join(session_path, 'trajectories_wo_gaps')
        trajs_path  = os.path.join(session_path, 'trajectories')

        if os.path.exists(session_path) and \
                (os.path.exists(blobs_path_1) or os.path.exists(blobs_path_2)) and \
                os.path.exists(vidobj_path):
            self._validation.enabled = True
        else:
            self._validation.enabled = False

        if os.path.exists(session_path) and \
                (os.path.exists(trajs_path) or os.path.exists(trajs_wo_path)) and \
                os.path.exists(vidobj_path):
            self._indiv_videos.enabled = True
            self._traj_video.enabled = True
        else:
            self._indiv_videos.enabled = False
            self._traj_video.enabled = False

    def __update_progress_evt(self, progress_count, max_count=None):
        if max_count is not None:
            self._progress.max = max_count
            self._progress.value = 0
            self._progress.show()
        elif self._progress.max == progress_count:
            self._progress.value = 0
        else:
            self._progress.value = progress_count

    def __open_videoannotator_evt(self):

        video_folder   = os.path.dirname(self.video_path)
        session_folder = "session_{0}".format(self._session.value)
        session_path   = os.path.join(video_folder, session_folder)

        subprocess.Popen([sys.executable,'-m', 'pythonvideoannotator', session_path])

    def __generate_individual_videos_evt(self):
        self._indiv_videos.enabled = False
        self._indiv_videos.label = self.GEN_INDIV_VIDEO_BTN_LABEL
        QApplication.processEvents()
        video_object, trajectories = get_video_object_and_trajectories(self.video_path, self._session.value)
        generate_individual_videos(video_object, trajectories)
        self._indiv_videos.enabled = True
        self._indiv_videos.label = self.INDIV_VIDEO_BTN_LABEL
        QApplication.processEvents()


    def __generate_trajectories_video_evt(self):
        self._traj_video.enabled = False
        self._traj_video.label = self.GEN_TRAJ_VIDEO_BTN_LABEL
        QApplication.processEvents()
        video_object, trajectories = get_video_object_and_trajectories(self.video_path, self._session.value)
        generate_trajectories_video(video_object, trajectories)
        self._traj_video.enabled = True
        self._traj_video.label = self.TRAJ_VIDEO_BTN_LABEL
        QApplication.processEvents()


    @property
    def open_multiple_files(self):
        return self._player.multiple_files


def get_video_object_and_trajectories(video_path, session_name):
    video_folder   = os.path.dirname(video_path)
    session_folder = "session_{0}".format(session_name)
    session_path   = os.path.join(video_folder, session_folder)
    trajs_wo_path  = os.path.join(session_path, 'trajectories_wo_gaps')
    trajs_path  = os.path.join(session_path, 'trajectories')

    video_object = np.load(os.path.join(session_path, 'video_object.npy'),
                           allow_pickle=True).item()

    if os.path.exists(trajs_wo_path):
        trajectories_file = glob.glob(os.path.join(trajs_wo_path, '*'))[-1]
    elif os.path.exists(trajs_path):
        trajectories_file = glob.glob(os.path.join(trajs_path, '*'))[-1]

    trajectories = np.load(trajectories_file, allow_pickle=True).item()['trajectories']

    return video_object, trajectories



import glob, os, cv2

from natsort import natsorted

class MultipleVideoCapture(object):


    def __init__(self, filepath):

        # VideoCapture objects
        self.captures = [cv2.VideoCapture(fn) for fn in self.search_files(filepath)]

        # Number of frames for each VideoCapture
        self.n_frames = [c.get(7) for c in self.captures]

        # Total frames ranges starting in 0
        self.frames_ranges = [0]

        # Active VideoCapture
        self.capture_index = 0

        for cap in self.captures:
            nframes = self.frames_ranges[-1]+int(cap.get(7))
            self.frames_ranges.append(nframes)

    # @classmethod
    # def search_files(cls, filepath):
    #     filedir = os.path.dirname(filepath)
    #     filename = os.path.basename(filepath)
    #     name, ext = os.path.splitext(filename)
    #
    #     if name.endswith('_1') or name.endswith('_01'):
    #         names = name.rsplit('_', 1)
    #         search_name = os.path.join(filedir, names[0] + '_*' + ext)
    #         print(sorted(glob.glob(search_name)))
    #         return sorted(glob.glob(search_name))
    #     else:
    #         return []

    @classmethod
    def search_files(cls, filepath):
        filedir = os.path.dirname(filepath)
        filename = os.path.basename(filepath)
        name, ext = os.path.splitext(filename)
        paths = glob.glob(os.path.join(filedir, '*' + ext))

        if len(paths) > 1:
            return natsorted(paths)
        else:
            return []

    @property
    def capture(self):
        return self.captures[self.capture_index]

    def read(self):
        next_capture = self.capture.get(1)==self.n_frames[self.capture_index] \
                    and self.capture_index < len(self.captures)-1
        if next_capture:
            self.capture_index += 1
            self.capture.set(1,0)
        res = self.capture.read()
        return res


    def get(self, flag):
        if flag==1:
            return self.frames_ranges[self.capture_index]+int(self.capture.get(1))
        elif flag==7:
            # Return the total of frames
            return self.frames_ranges[-1]
        else:
            return self.capture.get(flag)

    def set(self, flag, value):

        if flag==1:
            for index, nframes in enumerate(self.frames_ranges):
                if nframes<value:
                    self.capture_index = index

            self.capture.set(flag, value-self.frames_ranges[self.capture_index])
        else:
            self.capture.set(flag, value)




if __name__ == "__main__":

    video = MultipleVideoCapture('/home/ricardo/bitbucket/idtracker-project/idtrackerai_video_example_01.avi')



#!/usr/bin/python
# -*- coding: utf-8 -*-
'''
@author: Ricardo Ribeiro
@credits: Ricardo Ribeiro
@license: MIT
@version: 0.0
@maintainer: Ricardo Ribeiro
@email: ricardojvr@gmail.com
@status: Development
@lastEditedBy: Carlos Mão de Ferro (carlos.maodeferro@neuro.fchampalimaud.org)
'''
import pyforms_gui.utils.tools as tools
from pyforms_gui.controls.control_base import ControlBase
from AnyQt.QtWidgets import QLineEdit
from AnyQt 			 import uic

class ControlText(ControlBase):

	def init_form(self):
		control_path = tools.getFileInSameDirectory(__file__, "textInput.ui")
		self._form = uic.loadUi(control_path)
		self.form.label.setText(self._label)

		if self._value is not None: self.form.lineEdit.setText(self._value)
		
		self.form.setToolTip(self.help)

		super(ControlText, self).init_form()

		self.form.label.setAccessibleName('ControlText-label')
		self.form.lineEdit.editingFinished.connect(self.finishEditing)
		self.form.lineEdit.keyPressEvent = self.__key_pressed

	def finishEditing(self):
		"""Function called when the lineEdit widget is edited"""
		self.changed_event()

	def __key_pressed(self, event): 
		QLineEdit.keyPressEvent(self.form.lineEdit, event)

		self.key_pressed_event(event)

	def key_pressed_event(self, evt): pass
		
	###################################################################
	############ Properties ###########################################
	###################################################################

	@property
	def value(self):
		self._value = str(self._form.lineEdit.text())
		return self._value

	@value.setter
	def value(self, value):
		self._form.lineEdit.setText(value)
		ControlBase.value.fset(self, value)

	@property
	def label(self): return self.form.label.text()

	@label.setter
	def label(self, value):
		self.form.label.setText(value)
		ControlBase.label.fset(self, value)

	@property
	def readonly(self):
		return self._form.lineEdit.isReadOnly()

	@readonly.setter
	def readonly(self, value):
		self._form.lineEdit.setReadOnly(value)
		self._form.lineEdit.setEnabled(not value)



#!/usr/bin/python
# -*- coding: utf-8 -*-

__author__      = "Ricardo Ribeiro"
__credits__     = ["Ricardo Ribeiro"]
__license__     = "MIT"
__version__     = "0.0"
__maintainer__  = "Ricardo Ribeiro"
__email__       = "ricardojvr@gmail.com"
__status__      = "Development"



import time
from datetime import datetime as datetime_dt, timedelta

def timeit(method):
	def timed(*args, **kw):
		ts = time.time()
		result = method(*args, **kw)
		te = time.time()
		time_elapsed = datetime_dt(1,1,1) + timedelta(seconds=(te-ts) )
		print("%s: %d:%d:%d:%d;%d" % (method.__name__, time_elapsed.day-1, time_elapsed.hour, time_elapsed.minute, time_elapsed.second, time_elapsed.microsecond))
		return result
	return timed


import csv, time, datetime, decimal, logging, sys, numpy as np
from pyforms_gui.basewidget import BaseWidget
from pyforms_gui.controls.control_progress import ControlProgress
from pyforms_gui.controls.control_button import ControlButton
from pyforms_gui.controls.control_list import ControlList
from pyforms_gui.controls.control_number import ControlNumber
from pyforms_gui.controls.control_slider import ControlSlider
from pyforms_gui.controls.control_textarea import ControlTextArea
from pyforms_gui.controls.control_text import ControlText
from pyforms_gui.controls.control_number import ControlNumber
from pyforms_gui.controls.control_label import ControlLabel
from confapp import conf

from AnyQt.QtWidgets import QColorDialog

logger = logging.getLogger(__name__)

class GraphsProperties(BaseWidget):

	def __init__(self, timelineWidget=None, parent_win=None):
		super(GraphsProperties, self).__init__('Graphs properties', parent_win=parent_win)
		self.setContentsMargins(10, 10, 10, 10)
		self._mainwindow = parent_win
		self._timeline = timelineWidget

		#self.setMaximumWidth(300)


		# Definition of the forms fields
		self._graphs_list = ControlList('Graphs list')
		self._name        = ControlText('Name')
		self._min_value   = ControlNumber('Min', default=0,  minimum=-sys.float_info.max, maximum=sys.float_info.max)
		self._max_value   = ControlNumber('Max', default=0,  minimum=-sys.float_info.max, maximum=sys.float_info.max)
		self._values_zoom = ControlSlider('Amplitude', default=100,  minimum=60, maximum=400)
		self._values_top  = ControlNumber('Bottom', default=0,  minimum=-1000, maximum=1000)
		self._remove_graph_btn = ControlButton('Remove graph')
		self._value 	  = ControlLabel()
		self._pickcolor   = ControlButton('Pick color', default=self.__pickcolor_evt)

		self._graphs_list.readonly = True

		self._formset = [
			(
				['_graphs_list','_remove_graph_btn'],
				'||', 
				[
					' ',
					'_name',
					('_min_value', '_max_value', ' '),
					('_values_top', '_pickcolor'),
					'_values_zoom',
					'info:Choose one graph and move the mouse over \nthe timeline to visualize the coordinates.',
					'_value'
				]),
			]

		self._graphs_list.select_entire_row = True
		self._graphs_list.item_selection_changed_event = self.__graphs_list_selection_changed

		self._loaded = False
		self._current_selected_graph = None

		self._name.changed_event          = self.__save_graphs_changes
		self._min_value.changed_event     = self.__save_graphs_changes
		self._max_value.changed_event     = self.__save_graphs_changes
		self._values_zoom.changed_event   = self.__save_graphs_changes
		self._values_top.changed_event    = self.__save_graphs_changes

		self._name.enabled          = False
		self._min_value.enabled     = False
		self._max_value.enabled     = False
		self._values_zoom.enabled   = False
		self._values_top.enabled    = False
		self._remove_graph_btn.enabled = False
		self._pickcolor.enabled 		= False

		self._remove_graph_btn.value = self.__remove_chart


	def __add__(self, other):
		self._graphs_list += [other.name]
		return self

	def __sub__(self, other):
		self._graphs_list -= other
		return self

	def rename_graph(self, graph_index, newname):
		self._graphs_list.set_value(graph_index, 0, newname)

	@property
	def selected_graph(self):
		index = self._graphs_list.selected_row_index
		return self._timeline._graphs[index] if (index is not None) else None
	
	@property
	def coordinate_text(self): return self._value
	@coordinate_text.setter
	def coordinate_text(self, value):
		self._value.value = str(value) if value else ''
	


	def show(self):
		super(GraphsProperties, self).show()
		self._loaded = False

	def __pickcolor_evt(self):
		color = QColorDialog.getColor(
			self._current_selected_graph._color, 
			self, 'Pick a color for the graph')
		self._current_selected_graph._color = color
		self._timeline.repaint()
			

	def __remove_chart(self):
		index = self._graphs_list.selected_row_index
		if index is not None:
			self._current_selected_graph = None

			graph = self._timeline.graphs[index]
			graph.remove()

			self._loaded 					= False
			self._name.enabled          	= False
			self._min_value.enabled     	= False
			self._max_value.enabled     	= False
			self._values_zoom.enabled   	= False
			self._values_top.enabled    	= False
			self._remove_graph_btn.enabled 	= False
			self._pickcolor.enabled 		= False
			self._timeline.repaint()
			self._mainwindow -= index


	def __graphs_list_selection_changed(self):        
		graph = self.selected_graph
		self._updating_properties = True

		if graph is not None:
			graphmin = np.asscalar(graph.graph_min) if isinstance(graph.graph_min, np.generic) else graph.graph_min
			graphmax = np.asscalar(graph.graph_max) if isinstance(graph.graph_max, np.generic) else graph.graph_max
			
			exponent_min = abs(decimal.Decimal(graphmin).as_tuple().exponent)
			exponent_max = abs(decimal.Decimal(graphmax).as_tuple().exponent)
			exponent_min = 4 if exponent_min>4 else exponent_min
			exponent_max = 4 if exponent_min>4 else exponent_min

			self._name.value            = graph.name
			self._min_value.decimals    = exponent_min
			self._min_value.value       = graph.graph_min
			self._max_value.decimals    = exponent_max
			
			self._max_value.value   = graph.graph_max
			self._values_zoom.value = graph._zoom * 100.0
			self._values_top.value  = graph._top

			self._loaded 					= True
			self._name.enabled          	= True
			self._min_value.enabled     	= True
			self._max_value.enabled     	= True
			self._values_zoom.enabled   	= True
			self._values_top.enabled    	= True
			self._remove_graph_btn.enabled 	= True
			self._pickcolor.enabled 		= True
			self._current_selected_graph 	= graph

		del self._updating_properties
		


	def __save_graphs_changes(self):
		if hasattr(self, '_updating_properties'):  return

		if self._loaded and self._current_selected_graph is not None:
			graph = self._current_selected_graph

			#logger.debug('Before: Min: {0} | Max: {1} Zoom: {2}'.format(graph.graph_min, graph.graph_max,graph.zoom ) )

			graph.name = self._name.value; 
			graph.graph_min = self._min_value.value
			graph.graph_max = self._max_value.value

			graph.zoom     	= self._values_zoom.value / 100.0
			graph.top      	= self._values_top.value  

			logger.debug('Min: {0} | Max: {1} Zoom: {2}'.format(graph.graph_min, graph.graph_max,graph.zoom ) )

			self._timeline.repaint()

	@property
	def graphs(self):
		return self._timeline.graphs


	def mouse_moveover_timeline_event(self, event):
		graph = self.selected_graph
		if graph is not None: graph.mouse_move_evt(event, 0, self.height())
		
##################################################################################################################
##################################################################################################################
##################################################################################################################

# Execute the application
if __name__ == "__main__":
	import pyforms
	pyforms.start_app(GraphsProperties, geometry=(0, 0, 600, 400))



# !/usr/bin/python
# -*- coding: utf-8 -*-

import logging

from confapp import conf

logger = logging.getLogger(__name__)

from pyforms_gui.controls.control_base import ControlBase

from AnyQt.QtWidgets import QSizePolicy

from AnyQt import _api

from OpenGL.GL  import *
from OpenGL.GLU import *
        
from AnyQt import _api
if _api.USED_API == _api.QT_API_PYQT5:
    try:
        from AnyQt.QtOpenGL import QGLWidget
    except:
        logger.debug("No OpenGL library available")

    import platform
    if platform.system() == 'Darwin':
        from pyforms_gui.controls.control_player.VideoQt5GLWidget import VideoQt5GLWidget as VideoGLWidget
    else:
        from pyforms_gui.controls.control_player.VideoGLWidget import VideoGLWidget

elif _api.USED_API == _api.QT_API_PYQT4:
    try:
        from PyQt4.QtOpenGL import QGLWidget
    except:
        logger.debug("No OpenGL library available")

    from pyforms_gui.controls.control_player.VideoGLWidget import VideoGLWidget


class OpenglGLWidget(QGLWidget):
    def __init__(self, parent=None):
        QGLWidget.__init__(self, parent)

        self._zoom = 1.0
        self._scene = None
        self._rotation = [0, 0, 0]

        self._mouseLeftDown = False
        self._mouseRightDown = False

        self._mouseGLPosition = [0, 0, 0]
        self._lastMouseGLPosition = [0, 0, 0]

        self._mousePosition = [0, 0]  # Current mouse position
        self._lastMousePosition = [0, 0]  # Last mouse position

        self._mouseStartDragPoint = None
        self._clear_color = None

        self.setMinimumHeight(100)
        self.setMinimumWidth(100)
        self.setMouseTracking(True)
        self.setAcceptDrops(True)

        self.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding);

    def initializeGL(self):
        glClearDepth(1.0)
        if self._clear_color: glClearColor(*self._clear_color)
        glEnable(GL_BLEND)
        glBlendFunc(GL_SRC_ALPHA, GL_ONE)

        glEnable(GL_POINT_SMOOTH);
        glEnable(GL_LINE_SMOOTH);
        glEnable(GL_POLYGON_SMOOTH);
        glHint(GL_POINT_SMOOTH_HINT, GL_NICEST);
        glHint(GL_LINE_SMOOTH_HINT, GL_NICEST);
        glHint(GL_POLYGON_SMOOTH_HINT, GL_NICEST);

    def resizeGL(self, width, height):
        """
        
        :param width: 
        :param height: 
        :return: 
        """

        glViewport(0, 0, width, height)
        glMatrixMode(GL_PROJECTION)
        glLoadIdentity()

        if height!=0:
            ratio = float(width) / float(height)
        else:
            ratio = 1
        gluPerspective(65.0, ratio, 0.01, 800.0)

    def paintGL(self):
        """
        
        """
        if self._clear_color:
            glBlendFunc(GL_ONE_MINUS_CONSTANT_ALPHA, GL_ONE_MINUS_SRC_ALPHA)
            glClearColor(*self._clear_color)
        else:
            glBlendFunc(GL_SRC_ALPHA, GL_ONE)
            glClearColor(0, 0, 0, 1)

        glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)
        glMatrixMode(GL_MODELVIEW)
        glLoadIdentity()

        glEnable(GL_BLEND)

        glScalef(1, -1, -1)

        glTranslatef(0, 0, self._zoom)
        glRotatef(self._rotation[0], 1, 0, 0)
        glRotatef(self._rotation[1], 0, 1, 0)
        glRotatef(self._rotation[2], 0, 0, 1)

        if self._scene != None: self._scene.DrawGLScene()

        if self.mouseDown:
            # Get mouse position
            modelview, projection = glGetDoublev(GL_MODELVIEW_MATRIX), glGetDoublev(GL_PROJECTION_MATRIX)
            viewport = glGetIntegerv(GL_VIEWPORT)
            winX, winY = float(self._mousePosition[0]), float(viewport[3] - self._mousePosition[1])
            winZ = glReadPixels(winX, winY, 1, 1, GL_DEPTH_COMPONENT, GL_FLOAT)
            self._mouseGLPosition = gluUnProject(winX, winY, winZ[0][0], modelview, projection, viewport)

        glDisable(GL_BLEND)

    def __updateMouse(self, event, pressed=None):
        self._mousePosition[0] = event.x()
        self._mousePosition[1] = event.y()

        if pressed != None:
            if event.button() == 2: self._mouseRightDown = pressed
            if event.button() == 1: self._mouseLeftDown = pressed

    def wheelEvent(self, event):
        """
        
        :param event: 
        """
        if _api.USED_API == _api.QT_API_PYQT5:
            p = event.angleDelta()
            delta = p.y()
        elif _api.USED_API == _api.QT_API_PYQT4:
            delta = event.delta()
        
        
        if delta < 0:
            self._zoom += 1
        else:
            self._zoom -= 1

        self.repaint()

    def mouseReleaseEvent(self, event):
        """
        
        :param event: 
        """
        self.__updateMouse(event)

        if self._mouseRightDown:
            self._lastMouseGLPosition = self._mouseGLPosition
            self._mouseRightDown = False

        if self._mouseLeftDown:
            self.onEndDrag(self._mouseStartDragPoint, self._mouseGLPosition)
            self._mouseLeftDown = False

    def mousePressEvent(self, event):
        """
        
        :param event: 
        """
        QGLWidget.mousePressEvent(self, event)
        self.__updateMouse(event, pressed=True)
        self.repaint()

        if self._mouseLeftDown:  self._mouseStartDragPoint = self._mouseGLPosition
        if self._mouseRightDown: self._lastMouseGLPosition = self._mouseGLPosition

        self.onPress(event.button(), self._mousePosition, self._mouseGLPosition)

    def mouseMoveEvent(self, event):
        """
        
        :param event: 
        """
        QGLWidget.mouseMoveEvent(self, event)
        self.__updateMouse(event)
        self.repaint()

        self.onMove((self._mouseGLPosition[0], self._mouseGLPosition[1]))

        if self.mouseDown:
            if self._mouseLeftDown:
                # p = self._mouseGLPosition[0] - self._x, self._mouseGLPosition[1] + self._y
                # p = self._mouseGLPosition[0], self._mouseGLPosition[1]
                self.onDrag(self._lastMousePosition, self._mousePosition)

        self._lastMousePosition = list(self._mousePosition)

    def onMove(self, point):
        pass

    def onPress(self, button, point, glpoint=None):
        pass

    def onDrag(self, startPoint, endPoint, startGLPoint=None, endGLPoint=None):
        """
        
        :param startPoint: 
        :param endPoint: 
        :param startGLPoint: 
        :param endGLPoint: 
        """
        movX, movY = endPoint[0] - startPoint[0], endPoint[1] - startPoint[1]
        self._rotation[2] -= float(movX) * 0.15
        self._rotation[0] += float(movY) * 0.15

    def onEndDrag(self, startPoint, endPoint, startGLPoint=None, endGLPoint=None):
        pass

    ##############################################################################
    ###### Properties ############################################################
    ##############################################################################

    @property
    def scene(self):
        return self._scene

    @scene.setter
    def scene(self, value):
        self._scene = value

    @property
    def mouseDown(self):
        return self._mouseLeftDown or self._mouseRightDown

    def resetZoomAndRotation(self):
        self._zoom, self._rotation = 0.0, [0, 0, 0]


class ControlOpenGL(ControlBase):
    def init_form(self):
        self._form = OpenglGLWidget()
        super(ControlOpenGL, self).init_form()

    def repaint(self): self._form.repaint()

    def reset_zoom_and_rotation(self): self._form.resetZoomAndRotation()

    @property
    def value(self): return self._form.scene

    @value.setter
    def value(self, value):  self._form.scene = value; self._form.repaint()

    @property
    def clear_color(self): self._form._clear_color

    @clear_color.setter
    def clear_color(self, value):
        self._form._clear_color = value;
        self._form.repaint()

    @property
    def width(self): return self._form.width()

    @property
    def height(self): return self._form.height()



#!/usr/bin/python
# -*- coding: utf-8 -*-

from confapp import conf

from pyforms_gui.controls.control_base import ControlBase

import pyforms_gui.utils.tools as tools


from AnyQt           import uic, _api
from AnyQt.QtWidgets import QFileDialog


class ControlDir(ControlBase):

    def init_form(self):
        control_path = tools.getFileInSameDirectory(__file__, "fileInput.ui")
        self._form = uic.loadUi(control_path)
        self._form.pushButton.clicked.connect(self.click)
        self.form.label.setText(self._label)
        self.form.lineEdit.editingFinished.connect(self.finishEditing)
        self._form.pushButton.setIcon(conf.PYFORMS_ICON_FILE_OPEN)
        super(ControlDir, self).init_form()


    def click(self):
        if conf.PYFORMS_DIALOGS_OPTIONS:
            value = QFileDialog.getExistingDirectory(self.parent, self._label, options=conf.PYFORMS_DIALOGS_OPTIONS)
        else:
            value = QFileDialog.getExistingDirectory(self.parent, self._label)

        if value and len(value)>0: 
            self.value = value

    def finishEditing(self):
        """Function called when the lineEdit widget is edited"""
        self.changed_event()


    @property
    def value(self):
        self._value = str(self._form.lineEdit.text())
        return self._value

    @value.setter
    def value(self, value):
        self._form.lineEdit.setText(value)
        ControlBase.value.fset(self, value)

    @property
    def label(self): return self.form.label.text()

    @label.setter
    def label(self, value):
        self.form.label.setText(value)
        ControlBase.label.fset(self, value)



#!/usr/bin/python
# -*- coding: utf-8 -*-

""" pyforms_gui.controls.ControlPlayer.VideoGLWidget

"""
__author__      = "Ricardo Ribeiro"
__credits__     = ["Ricardo Ribeiro"]
__license__     = "MIT"
__version__     = "0.0"
__maintainer__  = "Ricardo Ribeiro"
__email__       = "ricardojvr@gmail.com"
__status__      = "Development"

import logging

from confapp     import conf
from AnyQt           import QtGui, QtCore, _api
from AnyQt.QtWidgets import QApplication

import OpenGL.GL  as GL
import OpenGL.GLU as GLU

 
try:
    import cv2
except:
    raise Exception('OpenCV is not available. Any Control using AbstractGLWidget will not be working')



class MouseEvent:

    def __init__(self, event):
        self.x = event.x()
        self.y = event.y()
        self.button = event.button()
        self.event = event


class AbstractGLWidget(object):

    def __init__(self, parent=None):
        super(AbstractGLWidget, self).__init__(parent)

        self.image_2_display = []
        self.textures = []
        self.setMouseTracking(True)
        self.zoom = 0.0
        self._mouseX = 0.0
        self._mouseY = 0.0
        
        #These variable are updated everytime the opengl scene is rendered 
        #and a mouse button is down
        self._glX = 0.0
        self._glY = 0.0
        self._glZ = 0.0

        #Last mouse opengl calculated position
        #This variable is updated everytime the opengl scene is rendered
        #and a mouse button is down
        self._mouse_pressed = False
        self._mouse_leftbtn_pressed = False

        self._mouse_clicked_event = None # store the event variable of the mouse click event
        self._mouse_dblclicked_event = None # store the event variable of the mouse double click event
        self._mouse_move_event = None # store the event variable of the mouse move event


        self._last_mouse_gl_pos = None

        self._lastGlX = 0.0 #Last 
        self._lastGlY = 0.0


        self._move_img  = False # flag used to move the image with the mouse click button
        self._width     = 1.0
        self._height    = 1.0
        self._x = 0
        self._y = 0
        self.img_width = 1
        self.img_height = 1

        self._rotateZ = 0
        self._rotateX = 0

        self._mouseStartDragPoint = None
        # Message to show on the left corner of the screen
        self._helpText = None

        self.setMinimumHeight(100)
        self.setMinimumWidth(100)

        self._point = None
        self._pending_frames = []

        self._tmp_msg = None

        self._font = QtGui.QFont()
        self._font.setPointSize(conf.PYFORMS_CONTROLPLAYER_FONT)


    def initializeGL(self):
        """
         Sets up the OpenGL rendering context, defines display lists, etc. 
         Gets called once before the first time resizeGL() or paintGL() is called.
        """
        GL.glClearDepth(1.0)
        GL.glClearColor(0, 0, 0, 1.0)
        GL.glEnable(GL.GL_DEPTH_TEST)

    def resizeGL(self, width, height):
        """
        Sets up the OpenGL viewport, projection, etc. 
        Gets called whenever the widget has been resized (and also when it is shown for 
        the first time because all newly created widgets get a resize event automatically).
        :param width:
        :param height:
        """
        GL.glViewport(0, 0, width, height)
        GL.glMatrixMode(GL.GL_PROJECTION)
        GL.glLoadIdentity()
        if height>0: GLU.gluPerspective(40.0, float(width) / float(height), 0.01, 10.0)

    def draw_video(self, width, height, x, y, z):
        # self.logger.debug("x: %s | y: %s | z: %s", x, y, z)
        GL.glPushMatrix()
        GL.glTranslatef(x, y, z)

        GL.glBegin(GL.GL_QUADS)
        GL.glTexCoord2f(0.0, 1.0)
        GL.glVertex3f(0, -height / 2.0, 0)  # top left
        GL.glTexCoord2f(0.0, 0.0)
        GL.glVertex3f(0, height / 2.0, 0)  # bottom left
        GL.glTexCoord2f(1.0, 0.0)
        GL.glVertex3f(width, height / 2.0, 0)  # bottom right
        GL.glTexCoord2f(1.0, 1.)
        GL.glVertex3f(width, -height / 2.0, 0)  # top right
        GL.glEnd()
        GL.glPopMatrix()

    def draw_pyramid(self, size=0.01):
        """Draw a multicolored pyramid"""
        GL.glBegin(GL.GL_TRIANGLES)
        GL.glVertex3f(0.0, size, 0.0)
        GL.glVertex3f(-size, -size, size)
        GL.glVertex3f(size, -size, size)
        GL.glVertex3f(0.0, size, 0.0)
        GL.glVertex3f(size, -size, size)
        GL.glVertex3f(size, -size, -size)
        GL.glVertex3f(0.0, size, 0.0)
        GL.glVertex3f(size, -size, -size)
        GL.glVertex3f(-size, -size, -size)
        GL.glVertex3f(0.0, size, 0.0)
        GL.glVertex3f(-size, -size, -size)
        GL.glVertex3f(-size, -size, size)
        GL.glEnd()

    def paintGL(self):
        """
        Renders the OpenGL scene. Gets called whenever the widget needs to be updated.
        """
        GL.glClearColor(0.5, 0.5, 0.5, 1.0)
        GL.glClear(GL.GL_COLOR_BUFFER_BIT | GL.GL_DEPTH_BUFFER_BIT)
        GL.glMatrixMode(GL.GL_MODELVIEW)
        GL.glLoadIdentity()
        
        # Correct a bug related with the overlap of contexts between simultaneous OpenGL windows.
        for index, frame in enumerate(self._pending_frames):

            color = GL.GL_LUMINANCE if len(frame.shape) == 2 else GL.GL_BGR
            w, h = len(frame[0]), len(frame) #Size of the image

            if len(self.textures)<len(self.image_2_display):
                self.textures.append(GL.glGenTextures(1))

            #Load the textures to opengl
            GL.glEnable(GL.GL_TEXTURE_2D)
            GL.glPixelStorei(GL.GL_UNPACK_ALIGNMENT, 1)
            GL.glBindTexture(GL.GL_TEXTURE_2D, self.textures[index])
            GL.glTexParameterf(GL.GL_TEXTURE_2D, GL.GL_TEXTURE_WRAP_S, GL.GL_CLAMP_TO_BORDER)
            GL.glTexParameterf(GL.GL_TEXTURE_2D, GL.GL_TEXTURE_WRAP_T, GL.GL_CLAMP_TO_BORDER)
            GL.glTexParameterf(GL.GL_TEXTURE_2D, GL.GL_TEXTURE_MAG_FILTER, GL.GL_LINEAR)
            GL.glTexParameterf(GL.GL_TEXTURE_2D, GL.GL_TEXTURE_MIN_FILTER, GL.GL_LINEAR)
            GL.glTexImage2D(GL.GL_TEXTURE_2D, 0, GL.GL_RGB, w, h, 0, color, GL.GL_UNSIGNED_BYTE, frame)

        self._pending_frames = []
        
        
        GL.glTranslatef(0, 0, -1)
        GL.glTranslatef(0, 0, -self.zoom)
        
        if len(self.image_2_display)>1: 
            #in case of having more images to display, it centers the images
            translate_x = float( (len(self.image_2_display)-1) * self._width) / 2.0
            GL.glTranslatef(-translate_x, 0, 0)

        if self._point is not None:
            GL.glColor4f(0, 0, 1, 1.0)
            GL.glPushMatrix()
            GL.glTranslatef(self._point[0], self._point[1], self._point[2])
            self.draw_pyramid()
            GL.glPopMatrix()
            GL.glColor4f(1, 1, 1, 1.0)

        GL.glRotatef(self._rotateX, -1, 0, 0)
        GL.glRotatef(self._rotateZ, 0, 0, 1)

        GL.glDisable(GL.GL_TEXTURE_2D)
        GL.glColor4f(0, 0, 0, .0)
        GL.glBegin(GL.GL_QUADS)
        GL.glVertex3f(20, -20, -.01)
        GL.glVertex3f(20, 20, -.001)
        GL.glVertex3f(-20, 20, -.001)
        GL.glVertex3f(-20, -20, -.001)
        GL.glEnd()

        GL.glColor4f(1, 1, 1, 1.0)

        # mouse events: find the image position where the mouse is
        if self._mouse_pressed:
            modelview   = GL.glGetDoublev(GL.GL_MODELVIEW_MATRIX)
            projection  = GL.glGetDoublev(GL.GL_PROJECTION_MATRIX)
            viewport    = GL.glGetIntegerv(GL.GL_VIEWPORT)
            winX = float(self._mouseX)
            winY = float(viewport[3] - self._mouseY)
            winZ = GL.glReadPixels( winX, winY, 1, 1, GL.GL_DEPTH_COMPONENT, GL.GL_FLOAT)
            self._glX, self._glY, self._glZ = GLU.gluUnProject( winX, winY, winZ[0][0], modelview, projection, viewport)

            if not self._last_mouse_gl_pos:
                self._last_mouse_gl_pos = self._glX, self._glY, self._glZ


            #mouse click event
            if self._mouse_clicked_event is not None:
            
                if hasattr(self, 'img_width'):
                    self.onClick(
                        self._mouse_clicked_event ,
                        self._get_current_x(),
                        self._get_current_y()
                    )

                if self._mouse_clicked_event.button == 1:
                    self._mouse_leftbtn_pressed = True
                    self._mouseStartDragPoint = self._get_current_mouse_point()

                if self._mouse_clicked_event.button == 4:
                    self._mouseStartDragPoint = self._get_current_mouse_point()
                    self._move_img = True
                    self._last_mouse_gl_pos = None
                    self._lastGlX = self._glX
                    self._lastGlY = self._glY
                self._mouse_clicked_event = None

            #mouse double click event
            if self._mouse_dblclicked_event is not None:
                if hasattr(self, 'img_width'):
                    self.onDoubleClick(self._mouse_dblclicked_event, self._get_current_x(), self._get_current_y())
                self._mouse_dblclicked_event = None

            #mouse move event
            if self._mouse_move_event is not None:
                if self._mouse_leftbtn_pressed and self._mouse_pressed:
                    p1 = self._mouseStartDragPoint
                    p2 = self._get_current_mouse_point()
                    self.onDrag(p1, p2)

                if self._move_img and self._mouse_pressed:
                    p1 = self._mouseStartDragPoint
                    p2 = self._get_current_mouse_point()
                    self.onDrag(p1, p2)
                self._mouse_move_event = None





        # end of the mouse events #################################


        GL.glEnable(GL.GL_TEXTURE_2D)
        GL.glDisable(GL.GL_DEPTH_TEST)

        if self._move_img and self._last_mouse_gl_pos is not None:
            self._x -= (self._last_mouse_gl_pos[0]-self._glX)
            self._y -= (self._last_mouse_gl_pos[1]-self._glY)

        for texture_index in range(0, len(self.image_2_display)):
            if texture_index>0: GL.glTranslatef(self._width, 0, 0)
            GL.glBindTexture(GL.GL_TEXTURE_2D, self.textures[texture_index])

            self.draw_video(self._width, self._height, self._x, self._y, 0.0)

        GL.glEnable(GL.GL_DEPTH_TEST)

        self.draw_helptext()
        self.draw_message()

        if self._move_img: 
            self._last_mouse_gl_pos = self._glX, self._glY, self._glZ

    def draw_helptext(self):
        if self._helpText is not None:
            self.qglColor(QtCore.Qt.black)
            self.renderText(5, 31, self._helpText, font=self._font)
            self.qglColor(QtCore.Qt.white)
            self.renderText(4, 30, self._helpText, font=self._font)

    def draw_message(self):
        if self._tmp_msg is not None:
            self.qglColor(QtCore.Qt.black)
            self.renderText(5, self.height()-19, self._tmp_msg, font=self._font)
            self.qglColor(QtCore.Qt.white)
            self.renderText(4, self.height()-20, self._tmp_msg, font=self._font)

    def reset(self):
        self.textures = []
        self._pending_frames = []
        self.image_2_display = []

    def show_tmp_msg(self, msg, timeout=2000):
        self._tmp_msg = msg
        self.update()
        QtCore.QTimer.singleShot(2000, self.__hide_tmp_msg )


    def paint(self, frames):
        if frames is None:
            self.reset()
            self.update()
            return
        elif self.image_2_display is None or len(self.image_2_display) == 0:
            self.img_height, self.img_width = frames[0].shape[:2]
            if self.img_width >= self.img_height:
                self._width = 1
                self._height = float(self.img_height) / float(self.img_width)
                self._x = -float(self._width) / 2
                self._y = 0
            else:
                self._height = 1
                self._width = float(self.img_width) / float(self.img_height)
                self._y = 0 
                self._x = -float(self._width) / 2 

        self.image_2_display = frames
        self._pending_frames = frames
        self.update()

    def wheelEvent(self, event):
        
        if not self._move_img:
            # Zoom the video
            self._mouseX = event.x()
            self._mouseY = event.y()

            if _api.USED_API == _api.QT_API_PYQT5:
                p = event.angleDelta()
                delta = p.y()
            elif _api.USED_API == _api.QT_API_PYQT4:
                delta = event.delta()
        
            zoom_factor = delta / float(1500)

            self.zoom += zoom_factor

            if self.zoom < -.98 and delta < 0:
                self.zoom = -0.98

            if self.zoom > 7 and delta > 0: # zoom limits
                self.zoom = 7

            # self.logger.debug("Wheel event | Current zoom: %s | Delta: %s | Zoom factor: %s", self.zoom, event.delta(), zoom_factor)
            self.update()

    
    def mouseReleaseEvent(self, event):

        self._mouse_pressed = False

        if event.button() == 4: self._move_img = False
            
        if event.button() == 1:
            if hasattr(self, 'img_width') and self._mouse_leftbtn_pressed:
                self.onEndDrag(self._mouseStartDragPoint, self._get_current_mouse_point())
                self._mouseStartDragPoint = None
            self._mouse_leftbtn_pressed = False

    
        
    def mousePressEvent(self, event):
        super(AbstractGLWidget, self).mousePressEvent(event)
        self.setFocus(QtCore.Qt.MouseFocusReason)

        self._mouse_pressed = True
        self._mouseX = event.x()
        self._mouseY = event.y()
        self._mouse_clicked_event = MouseEvent(event)

        self.repaint()
        
    def mouseDoubleClickEvent(self, event):
        self._mouse_pressed = True
        self._mouseX = event.x()
        self._mouseY = event.y()
        self._mouse_dblclicked_event = MouseEvent(event)

        self.repaint()
        

    def mouseMoveEvent(self, event):
        self.setFocus(QtCore.Qt.MouseFocusReason)

        self._mouseX = event.x()
        self._mouseY = event.y()
        self._mouse_move_event = MouseEvent(event)

        QApplication.processEvents()
        self.update()
        

    def keyPressEvent(self, event):
        super(AbstractGLWidget, self).keyPressEvent(event)

        #Set the flag move_img to true, for the image position to be updated
        if event.key() == QtCore.Qt.Key_M:
            self._move_img = True

        self.on_key_press(event)


    def keyReleaseEvent(self, event):
        super(AbstractGLWidget, self).keyReleaseEvent(event)

        if event.key() == QtCore.Qt.Key_M:
            self._move_img = False

        self.on_key_release(event)


    def __hide_tmp_msg(self): 
        self._tmp_msg = None
        self.update()

    def onDoubleClick(self, event, x, y): pass

    def onClick(self, event, x, y): pass

    def onDrag(self, startPoint, endPoint): pass

    def onEndDrag(self, startPoint, endPoint): pass

    def on_key_release(self, event):
        # Control video playback using the space bar to Play/Pause
        if event.key() == QtCore.Qt.Key_Space:

            if self._control.is_playing:
                self._control.stop()
            else:
                self._control.play()

        # Jumps 1 frame backwards
        elif event.key() == QtCore.Qt.Key_A:
            self._control.video_index -= 2
            self._control.call_next_frame()

        # Jumps 1 frame forward
        elif event.key() == QtCore.Qt.Key_D:
            self._control.call_next_frame()

        # Jumps 20 seconds backwards
        elif event.key() == QtCore.Qt.Key_Z:
            self._control.jump_backward()
            self._control.call_next_frame()

        # Jumps 20 seconds forward
        elif event.key() == QtCore.Qt.Key_C:
            self._control.jump_forward()
            self._control.call_next_frame()

        elif event.key() == QtCore.Qt.Key_1:
            self._control.next_frame_step = 1
            self.show_tmp_msg('Speed: 1x')

        elif event.key() == QtCore.Qt.Key_2:
            self._control.next_frame_step = 2
            self.show_tmp_msg('Speed: 2x')

        elif event.key() == QtCore.Qt.Key_3:
            self._control.next_frame_step = 3
            self.show_tmp_msg('Speed: 3x')

        elif event.key() == QtCore.Qt.Key_4:
            self._control.next_frame_step = 4
            self.show_tmp_msg('Speed: 4x')

        elif event.key() == QtCore.Qt.Key_5:
            self._control.next_frame_step = 5
            self.show_tmp_msg('Speed: 5x')

        elif event.key() == QtCore.Qt.Key_6:
            self._control.next_frame_step = 6
            self.show_tmp_msg('Speed: 6x')

        elif event.key() == QtCore.Qt.Key_7:
            self._control.next_frame_step = 7
            self.show_tmp_msg('Speed: 7x')

        elif event.key() == QtCore.Qt.Key_8:
            self._control.next_frame_step = 8
            self.show_tmp_msg('Speed: 8x')

        elif event.key() == QtCore.Qt.Key_9:
            self._control.next_frame_step = 9
            self.show_tmp_msg('Speed: 9x')

    def on_key_press(self, event): pass


    ##########################################################################
    ############ PROPERTIES ##################################################
    ##########################################################################

    @property
    def control(self): return self._control

    @property
    def rotateX(self): return self._rotateX

    @rotateX.setter
    def rotateX(self, value):
        self._rotateX = value
        self.update()

    @property
    def rotateZ(self): return self._rotateZ

    @rotateZ.setter
    def rotateZ(self, value):
        self._rotateZ = value
        self.update()

    def _get_current_mouse_point(self):
        """

        """
        return self._get_current_x(), self._get_current_y()

    def _get_current_x(self):
        if self.img_width>self.img_height:
            return (self._glX - self._x) * float(self.img_width)
        else:
            return (self._glX - self._x) * float(self.img_height)

    def _get_current_y(self):
        if self.img_width > self.img_height:
            return (self._height - self._glY + self._y) * float(self.img_width) - self.img_height / 2.0
        else:
            return (self._height - self._glY + self._y) * float(self.img_height) - self.img_height / 2.0


    @property
    def point(self): return self._point

    @point.setter
    def point(self, value):
        if hasattr(self, 'img_width'):
            x = value[0] / float(self.img_width)  # +self._x
            y = -value[1] / float(self.img_width)  # -self._y-self._height)
            z = 0.1  # value[2]
            self._point = x, y, z


#!/usr/bin/python
# -*- coding: utf-8 -*-

""" pyforms_gui.controls.ControlEventTimeline.__init__

"""


__author__ = ["Ricardo Ribeiro", "Hugo Cachitas"]
__credits__ = ["Ricardo Ribeiro", "Hugo Cachitas"]
__license__ = "MIT"
__version__ = "0.0"
__maintainer__ = "Ricardo Ribeiro"
__email__ = "ricardojvr@gmail.com"
__status__ = "Development"



# !/usr/bin/python
# -*- coding: utf-8 -*-

from AnyQt import QtCore
from AnyQt.QtWidgets import QWidget, QMessageBox
from AnyQt.QtGui import QColor, QPainter, QFont, QCursor, QKeyEvent
from pyforms_gui.controls.control_event_timeline.events.track import Track
from pyforms_gui.controls.control_event_timeline.events.event import Event
from pyforms_gui.controls.control_event_timeline.graphs.graph import Graph
from pyforms_gui.controls.control_event_timeline.events.pointer import Pointer

class TimelineWidget(QWidget):
    """
    Timeline widget definition to be used in the ControlEventTimeline
    """
    TOPTRACK_HEIGHT     = 20  # Space in pixels of the top frames track.
    TRACK_HEIGHT        = 34  # Track height in pixels.
    TRACK_TITLE_TOP_POS = 12  # Relative position of the title of the track in pixels.

    EVENT_HEIGHT        = 30  # Height of each event in pixels.
    EVENT_TITLE_TOP_POS = 19  # Relative position of the title of each event in pixels.
    EVENT_RANGE_TOP_POS = 44  # Relative position of the range of each event in pixels.

    GRAPHS_COLORS = [
        QColor(240, 163, 255), QColor(0, 117, 220), QColor(153, 63, 0), QColor(76, 0, 92),
        QColor(25, 25, 25), QColor(0, 92, 49), QColor(43, 206, 72), QColor(255, 204, 153),
        QColor(128, 128, 128), QColor(148, 255, 181), QColor(143, 124, 0), QColor(157, 204, 0),
        QColor(194, 0, 136), QColor(0, 51, 128), QColor(255, 164, 5), QColor(255, 168, 187),
        QColor(66, 102, 0), QColor(255, 0, 16), QColor(94, 241, 242), QColor(0, 153, 143),
        QColor(116, 10, 255), QColor(153, 0, 0), QColor(255, 255, 0), QColor(255, 80, 5)
    ]  # List of colors to be used in new graphs


    def __init__(self, control):
        """
        Timeline widget
        :param control: object of the Pyforms control.
        """
        super(TimelineWidget, self).__init__()

        self.setMouseTracking(True)
        self.setMinimumWidth(300000)

        # timeline background color
        palette = self.palette()
        palette.setColor(self.backgroundRole(), QtCore.Qt.white)
        self.setPalette(palette)

        self.control = control  # parent control

        self._defautcolor = QColor(100, 100, 255)  # default events color


        self._graphs = []  # list of graphs
        self._tracks = []  # list of tracks

        self._scale = 1.0               # scale of the timeline
        self._last_mousex = None        # last know X coordinate of the mouse. Used for mouse drag.
        self._mouse_current_pos = None  # mouse current position

        self._moving = False          # flag: moving an event.
        self._resizing_began = False  # flag: resize of an event is active.
        self._resizing_ended = False  # flag: resize of an event ended.
        self._creating_event = False  # flag: event is being created
        self._creating_event_start = None  # flag: the S key pressed to create an event is active
        self._creating_event_end = None    # flag: the S key was pressed to finish the event

        self._selected = None             # selected event.
        self._selected_track = None       # selected track.
        self._pointer = Pointer(0, self)  # timeline pointer.

        # Video playback controls
        self._video_playing = False
        self._video_fps = None
        self._video_fps_min = None
        self._video_fps_max = None
        self._video_fps_inc = None

        self._repainting = False

    ##########################################################################
    #### HELPERS/FUNCTIONS ###################################################
    ##########################################################################

    def __add__(self, other):
        if isinstance(other, Track):
            self._tracks.append( other )
        else:
            self.control.__add__(other)
        return self

    def __sub__(self, other):
        if isinstance(other, Track):
            if self.selected_row == other:
                self.selected_row = None
            self._tracks.remove( other )
            for i, track in enumerate(self._tracks):
                track.index = i
            self.setMinimumHeight(Track.which_top(len(self._tracks)))
        elif isinstance(other, Graph):
            self._graphs.remove(other)
            self.repaint()
        else:
            self.control.__sub__(other)
        return self

    def open_and_close_event(self):

        if not self._creating_event:
            # Start
            self._creating_event_start = self._pointer.frame
            self._creating_event = True

            # TODO Add some indicator that an event is being recorded, like
            # using the track selector circle to become red

        elif self._creating_event:
            # End, must be followed right after Start key and have no
            # effect otherwise
            self._creating_event_end = self._pointer.frame

            start = self._creating_event_start
            end   = self._creating_event_end
            comment = ""

            if end > start:
                track = self._selected_track
                if track is None and len(self.tracks) > 0:
                    track = self.tracks[0]
                if track is None:
                    track = self.add_track()

                self.add_event(start, end, comment, track=track)
                self.repaint()

            self._creating_event = False


    def close_event(self):
        pass


    def add_graph(self, name, data):
        """
        Add a graph to the timeline.
        :param str name: Name of the graph
        :param list(int) data: List the size of the video with the graph value in each position.
        :return: Created graph.
        """
        graph = Graph(self, name=name, color=self.GRAPHS_COLORS[len(self._graphs)])
        graph.import_data(data)
        self._graphs.append(graph)
        self.repaint()
        return graph

    def add_track(self, title='', color=None):
        """
        Add a new track.
        :param str title: Title of the track.
        :param QColor color: Default color of the events in the track.
        :return: Return the added track.
        """
        t = Track(self, title=title, color=color)
        if self.selected_row is not None:
            index = self.selected_row.index
            self._tracks.insert(index, t)
            for i, track in enumerate(self.tracks):
                track.index = i
        else:
            self._tracks.append(t)
        self.setMinimumHeight(Track.which_top(len(self._tracks)))
        return t

    def get_track(self, title):
        """
        Get a track by its title
        :param str title: Title of the track.
        :return: Return the track with the matching title.
        """
        for track in self.tracks:
            if track.title==title:
                return track
        return None

    def add_event(self, begin, end, title='', track=None, lock=False, color=None, row=0):
        """
        Add a new event to the timeline.
        :param int begin: Initial frame of the event.
        :param int end: Final frame of the event.
        :param str title: Title of the event.
        :param Track track: Track object of the event.
        :param bool lock: Flag indicating if the event is lock or not.
        :param QColor color: Color of the event.
        :return: Return the created Event object.
        """
        if track is None:
            for i in range( len(self._tracks), row+1):
                self.add_track()
            track = self._tracks[row]
        return Event( begin, end, title=title, lock=lock, color=color, track=track, widget=self)

    def remove_selected_event(self):
        """
        Remove the selected event from the timeline.
        """
        if self._selected is not None and not self._selected.lock:
            self._selected.remove()
            self._selected = None
            self.repaint()

    def remove_selected_track(self):
        """
        Remove the selected track from the timeline.
        """
        if self._selected_track is not None:
            self._selected_track.remove()
            self.repaint()

    def move_track_up(self, track):
        """
        Move a track up
        :param Track track: Track to move
        """
        if track.index<=0: return

        i = track.index
        self._tracks[i-1], self._tracks[i] = self._tracks[i], self._tracks[i-1]
        for i, track in enumerate(self._tracks):
            track.index = i
        self.repaint()

    def move_track_down(self, track):
        """
        Move a track down
        :param Track track: Track to move
        """
        if track.index>=(len(self.tracks)-1): return

        i = track.index
        self._tracks[i+1], self._tracks[i] = self._tracks[i], self._tracks[i+1]
        for i, track in enumerate(self._tracks):
            track.index = i
        self.repaint()

    def toggle_selected_event_lock(self):
        """
        Toggle lock of the selected event.
        """
        if self._selected is not None:
            self._selected.lock = not self._selected.lock
            self.repaint()




    def clean(self):
        """
        Clean everything in the timeline
        """
        # remove tracks
        for i in range( len(self.tracks)-1 , -1, -1):
            track = self.tracks[i]
            track.remove()

        # remove graphs
        for i in range( len(self.graphs)-1 , -1, -1):
            graph = self.graphs[i]
            graph.remove()

        self.repaint()



    def find_track(self, ycoord):
        """
        Find a track.
        :param int ycoord: Y coordinate in pixels of the track to find.
        :return: Track object
        """
        if ycoord<=self.TOPTRACK_HEIGHT: return self._selected_track

        track_index = (ycoord - self.TOPTRACK_HEIGHT) // self.TRACK_HEIGHT
        ntracks 	= len(self.tracks)

        # if the track does not exists yet, create it
        if track_index >= ntracks:
            for i in range(ntracks, track_index+1):
                self.add_track()

        return self.tracks[track_index]



    def select_event(self, xcoord, ycoord):
        """
        Select an event based on the x and y pixel coordinates.
        :param int xcoord: X pixel coordinate.
        :param int ycoord: Y pixel coordinate.
        :return: Return the selected event.
        """
        # Check if the timeline pointer was selected
        if ycoord <= self.TOPTRACK_HEIGHT:
            return self._pointer if self._pointer.collide(xcoord, ycoord) else None

        # Check if the timeline events were selected
        i = Track.whichTrack(ycoord)
        if i >= len(self._tracks):
            return None

        return self._tracks[i].select_event(xcoord, ycoord)


    def x2frame(self, x):
        """
        Convert X pixel coordinate to a video frame number.
        :param int x: X coordinate in pixels.
        :return: Number of the correspondent frame.
        """
        return int(x / self._scale)

    def frame2x(self, frame):
        """
        Convert a video frame number to X pixel coordinate in the widget.
        :param int frame: Frame number in the video.
        :return: X coordinate of the correspondent pixel.
        """
        return int(frame * self._scale)




    ##########################################################################
    #### IO Functions ########################################################
    ##########################################################################

    def import_events_from_csvreader(self, csvreader):
        """
        Import events from a csv.reader object.
        :param csv.reader csvreader: csv.reader object from where the data will be imported.
        """
        self._selected = None

        track = self.tracks[-1] if len(self.tracks)>0 else None
        for row in csvreader:
            if len(row) == 0:
                continue

            if row[0] == "T":
                track = self.add_track( title = row[1], color = QColor(row[2]) )

            elif row[0] == "P":
                self.add_event( int(row[2]), int(row[3]), title=row[4], color=QColor(row[5]), lock=(row[1]=='True'), track=track)


    def export_events_to_csvwriter(self, csvwriter):
        """
        Export events to a csv.writer object.
        :param csv.writer csvwriter: csv.writer object to where the data will be exported.

        Current file structure:
        =======================

        --- CSV FILE BEGIN ---
        Track info line
        Event info line
        Event info line
        ...
        Track info line
        Event info line
        Event info line
        ...
        Track info line
        Event info line
        Event info line
        ...
        --- CSV FILE END ---

        Track info line format:
        =======================
        | T | Total # of events in this track |  |  | Color | Label |

        Event info line format:
        =======================
        | P | Lock status | Begin frame | End frame | Comment | Color |s
        """
        for index, track in enumerate(self._tracks):
            csvwriter.writerow(track.properties)
            for delta in track.events:
                csvwriter.writerow(delta.properties)


    def exportmatrix_events_to_csvwriter(self, csvwriter):
        """
        Export events to a csv.writer object in a matrix format.
        :param csv.writer csvwriter: csv.writer object to where the data will be exported.
        """
        for index, track in enumerate(self._tracks):
            _, track_title, _ = track.properties
            for delta in track.events:
                _, _, begin, end, delta_title, _, _ = delta.properties
                row = [track_title, begin, end, delta_title]
                csvwriter.writerow(row)


    ##########################################################################
    #### EVENTS ##############################################################
    ##########################################################################

    def paintEvent(self, event):
        """
        Paint event
        :param event:
        """
        super(TimelineWidget, self).paintEvent(event)

        painter = QPainter()
        painter.begin(self)

        painter.setRenderHint(QPainter.Antialiasing)
        painter.setFont(QFont('Decorative', 8))

        # find the start and end X coordinate to draw
        start = self.scrollbar.sliderPosition()
        end = start + self.parent().width() + 50

        # Draw graphs ##########################################################
        if len(self._graphs) > 0:
            painter.setPen(QtCore.Qt.black)
            middle = self.height() // 2
            painter.setOpacity(0.1)
            painter.drawLine(start, middle, end, middle)

        for chart in self._graphs:
            chart.draw(painter, start, end, 0, self.height())
        # End draw graph #######################################################

        for track in self.tracks:
            track.draw_title(painter, start, end)

        self.__draw_track_lines(painter, start, end)

        if self._selected_track is not None:
            self._selected_track.draw_background(painter, start, end)

        for track in self.tracks:
            track.draw_events(painter, start, end)

        # Draw the selected element
        if self._selected != None:
            painter.setBrush(QColor(255, 0, 0))
            self._selected.draw(painter, showvalues=True)

        # Draw the time pointer
        self._pointer.draw(painter, highlight=self._creating_event)
        painter.end()

        if not hasattr(self, '_is_refreshing'):
            self._is_refreshing = True
            self.update()
        else:
            del self._is_refreshing


    def mouseDoubleClickEvent(self, event):
        """

        :param event:
        :return:
        """
        xcoord = event.x()
        ycoord = event.y()

        if  self._selected is not None and \
                self._selected != self._pointer and \
                self._selected.collide( xcoord, ycoord ):
            # check if a period event is selected,
            # if so open the properties window
            self._selected.open_properties()

        elif ycoord > self.TOPTRACK_HEIGHT:
            # check if click was bellow the time track
            # if so create a new event
            track = self.find_track( ycoord )
            x     = xcoord / self._scale
            time  = Event(x, x + 10, title='', track=track, widget=self)
            self._selected = time
            self._selected_track = track
            self.repaint()

    def key_release_event(self, event):
        if self._selected is not None:
            modifier = int(event.modifiers())

            # Move the event (or the pointer) left
            if modifier == QtCore.Qt.ControlModifier and event.key() == QtCore.Qt.Key_Left:
                self._selected.move(-1, 0)
                event.ignore()
                self.repaint()

            # Move the event (or the pointer) right
            if modifier == QtCore.Qt.ControlModifier and event.key() == QtCore.Qt.Key_Right:
                self._selected.move(1, 0)
                event.ignore()
                self.repaint()

            if self._selected != self._pointer:
                # Delete the selected event
                if event.key() == QtCore.Qt.Key_Delete:
                    self.remove_selected_event()

                # Lock or unlock an event
                if event.key() == QtCore.Qt.Key_L:
                    self.toggle_selected_event_lock()

                # Move to the next event
                if event.key() == QtCore.Qt.Key_E:
                    index = self.selected_row.events.index(self._selected)
                    if index < len(self.selected_row.events)-1:
                        self._selected = self.selected_row.events[index+1]
                        self.position = self._selected.begin

                # Move to the previous event
                if event.key() == QtCore.Qt.Key_Q:
                    index = self.selected_row.events.index(self._selected)
                    if index > 0:
                        self._selected = self.selected_row.events[index - 1]
                        self.position = self._selected.begin

                # Move the event up
                if modifier == QtCore.Qt.ControlModifier and event.key() == QtCore.Qt.Key_Up:
                    self._selected.move(0, self._selected.top_coordinate - self.TRACK_HEIGHT)
                    self.repaint()

                # Move the event down
                if modifier == QtCore.Qt.ControlModifier and event.key() == QtCore.Qt.Key_Down:
                    self._selected.move(0, self._selected.top_coordinate + self.TRACK_HEIGHT)
                    self.repaint()

                # Move the event end left
                if modifier == int(
                        QtCore.Qt.ShiftModifier | QtCore.Qt.ControlModifier) and event.key() == QtCore.Qt.Key_Left:
                    self._selected.move_end(-1)
                    self.repaint()

                # Move the event end right
                if modifier == int(
                        QtCore.Qt.ShiftModifier | QtCore.Qt.ControlModifier) and event.key() == QtCore.Qt.Key_Right:
                    self._selected.move_end(1)
                    self.repaint()

                # Move the event begin left
                if modifier == QtCore.Qt.ShiftModifier and event.key() == QtCore.Qt.Key_Left:
                    self._selected.move_begin(-1)
                    self.repaint()

                # Move the event begin right
                if modifier == QtCore.Qt.ShiftModifier and event.key() == QtCore.Qt.Key_Right:
                    self._selected.move_begin(1)
                    self.repaint()

        else:
            # Keybinds to create an event at current frame
            if event.key() == QtCore.Qt.Key_S:
                self.open_and_close_event()


            # walk backwards 1 step
            elif event.key() == QtCore.Qt.Key_A:
                self.position = self.position - 1

            # forward 1 step
            elif event.key() == QtCore.Qt.Key_D:
                self.position = self.position + 1

            # Move to the first event
            elif event.key() == QtCore.Qt.Key_E:
                if self.selected_row is not None and len(self.selected_row)>0:
                    self._selected = self.selected_row.events[0]
                    self.position = self._selected.begin

            # Move to the last event
            elif event.key() == QtCore.Qt.Key_Q:
                if self.selected_row is not None and len(self.selected_row)>0:
                    self._selected = self.selected_row.events[len(self.selected_row)-1]
                    self.position = self._selected.begin

    def keyReleaseEvent(self, event:  QKeyEvent):
        super(TimelineWidget, self).keyReleaseEvent(event)

        self.key_release_event(event)

    def keyPressEvent(self, event: QKeyEvent):
        event.ignore()
        pass


    def mousePressEvent(self, event):
        """
        Event called when the mouse buttons are pressed
        :param event: Mouse event
        """

        xcoord = event.x()
        ycoord = event.y()

        self._selected_track = self.find_track(ycoord)

        # Select the period bar
        self._selected = self.select_event(xcoord, ycoord)
        self._moving   = False
        self._resizing_began = False
        self._resizing_ended = False

        if self._selected is not None:
            # if no event is selected

            if event.buttons() == QtCore.Qt.LeftButton:
                # check the action to execute

                if self._selected.can_slide_end(xcoord, ycoord):
                    # Resize the event at the end
                    self._resizing_ended = True

                elif self._selected.can_slide_begin(xcoord, ycoord):
                    # Resize the event at the beginning
                    self._resizing_began = True

                elif self._selected.collide(xcoord, ycoord):
                    # Move the period
                    self._moving = True

        if ycoord <= self.TOPTRACK_HEIGHT and not self._moving:
            # move the time pointer
            self._pointer.position = self.x2frame(xcoord)

        self.repaint()

    def mouseMoveEvent(self, event):
        """
        Event called when the mouse is moved
        :param event: Mouse move event.
        """
        super(TimelineWidget, self).mouseMoveEvent(event)
        self.control.mouse_moveover_timeline_event(event)

        xcoord, ycoord = event.x(), event.y()
        self._mouse_current_pos = xcoord, ycoord

        # Do nothing if no event bar is selected
        if self._selected is None:
            return

        # set cursors
        if  self._selected.can_slide_begin(xcoord, ycoord) or \
                self._selected.can_slide_end(xcoord, ycoord):
            # resize cursor.
            self.setCursor(QCursor(QtCore.Qt.SizeHorCursor))

        elif self._selected.collide(xcoord, ycoord):
            # move cursor
            self.setCursor(QCursor(QtCore.Qt.SizeAllCursor))

        else:
            # normal cursor
            self.setCursor(QCursor(QtCore.Qt.ArrowCursor))

        if event.buttons() == QtCore.Qt.LeftButton:
            # move the period
            if self._last_mousex is not None:
                diff = xcoord - self._last_mousex
                if diff != 0:
                    if self._moving:
                        # move the selected period
                        self._selected.move(diff, ycoord)

                    elif self._resizing_began:
                        # resize the beginning of the period
                        self._selected.move_begin(diff)

                    elif self._resizing_ended:
                        # resize the end of the period
                        self._selected.move_end(diff)

                    self.repaint()
            self._last_mousex = xcoord



    def mouseReleaseEvent(self, event):
        """
        Event called when the mouse is released
        :param event: Mouse event
        """
        self._last_mousex = None


    ##########################################################################
    #### PRIVATE FUNTIONS ####################################################
    ##########################################################################

    def __draw_track_lines(self, painter, start, end):
        """
        Draw the track lines.
        :param QPainter painter: Qt QPainter object.
        :param int start: Initial X coordinate pixel to paint.
        :param int end: Final X coordinate pixel to paint.
        """
        # Draw only from pixel start to end
        painter.setPen(QtCore.Qt.DashLine)
        painter.setOpacity(0.3)
        for i, track in enumerate(self._tracks):
            track.draw(painter, start, end)

        # Draw vertical lines
        for x in range(start - (start % 100), end, 100):
            painter.drawLine(x, self.TOPTRACK_HEIGHT, x, self.height())
            string = "{0}".format(int(round(x / self._scale)))
            boundtext = painter.boundingRect(QtCore.QRectF(), string)
            painter.drawText(x - boundtext.width() / 2, 15, string)

            if self._video_fps:
                string = "{0}".format(int(round((x / self._scale) * (1000.0 / self._video_fps))))
                boundtext = painter.boundingRect(QtCore.QRectF(), string)
                painter.drawText(x - boundtext.width() / 2, 30, string)

        painter.setOpacity(1.0)


    def __set_frame_visible(self,  frame_index):
        """
        Set the frame index visible.
        :param int frame_index: Video frame index.
        """
        xcoord = self.frame2x(frame_index)
        scroll_limit = self.scrollbar.sliderPosition() + self.parent().width()-50

        if xcoord > scroll_limit:
            self.scrollbar.setSliderPosition(xcoord - self.parent().width() + 50)

        elif xcoord < self.scrollbar.sliderPosition():
            self.scrollbar.setSliderPosition(xcoord)

    ##########################################################################
    #### PROPERTIES ##########################################################
    ##########################################################################

    @property
    def scrollbar(self):
        """
        :return: The horizontal scrollbar.
        """
        return self._scroll.horizontalScrollBar()

    @property
    def position(self):
        """
        :return: The current position of the timeline.
        """
        return self._pointer._position

    @position.setter
    def position(self, value):
        self._pointer.position = value
        # Check if the player position is inside the scroll
        # if is not in, update the scroll position
        self.__set_frame_visible(value)
        self.repaint()

    @property
    def scale(self):
        """
        :return: Scale of the timeline.
        """
        return self._scale

    @scale.setter
    def scale(self, value):
        self._scale = value
        self.repaint()

    @property
    def color(self):
        """
        :return: Default color of the track events.
        """
        return self._defautcolor

    @color.setter
    def color(self, value):
        self._defautcolor = value

    @property
    def tracks(self):
        """
        :return: List of tracks in the timeline.
        """
        return self._tracks

    @property
    def graphs(self):
        """
        :return: List of graphs in the timeline.
        """
        return self._graphs

    @property
    def is_playing(self):
        return self._video_playing

    @property
    def fps(self):
        return self._video_fps

    @fps.setter
    def fps(self, value):
        self._video_fps = value

    @property
    def graphs_properties(self):
        return self.control._graphs_prop_win

    @property
    def current_mouseover_track(self):
        """
        :return: Return the track where the mouse is over.
        """
        if self._mouse_current_pos is None:
            return None
        return (self._mouse_current_pos[1] - self.EVENT_HEIGHT) // self.TRACK_HEIGHT

        self._moving = False          # flag: moving an event.
        self._resizing_began = False  # flag: resize of an event is active.
        self._resizing_ended = False  # flag: resize of an event ended.
        self._creating_event = False  # flag: event is being created
        self._creating_event_start = None  # flag: the S key pressed to create an event is active
        self._creating_event_end = None    # flag: the S key was pressed to finish the event

        self._selected = None             # selected event.
        self._selected_track = None       # selected track.
        self._pointer = Pointer(0, self)  # timeline pointer.

        # Video playback controls
        self._video_playing = False
        self._video_fps = None
        self._video_fps_min = None
        self._video_fps_max = None
        self._video_fps_inc = None

    @property
    def selected(self):
        return self._selected

    @selected.setter
    def selected(self, value):
        self._selected = value

    @property
    def pointer(self):
        return self._pointer

    @pointer.setter
    def pointer(self, value):
        self._pointer = value
    
    @property
    def moving(self):
        return self._moving

    @moving.setter
    def moving(self, value):
        self._moving = value

    @property
    def resizing_began(self):
        return self._resizing_began

    @resizing_began.setter
    def resizing_began(self, value):
        self._resizing_began = value

    @property
    def resizing_ended(self):
        return self._resizing_ended

    @resizing_ended.setter
    def resizing_ended(self, value):
        self._resizing_ended = value

    @property
    def creating_event(self):
        return self._creating_event

    @creating_event.setter
    def creating_event(self, value):
        self._creating_event = value

    @property
    def creating_event_start(self):
        return self._creating_event_start

    @creating_event_start.setter
    def creating_event_start(self, value):
        self._creating_event_start = value

    @property
    def creating_event_end(self):
        return self._creating_event_end

    @creating_event_end.setter
    def creating_event_end(self, value):
        self._creating_event_end = value

    @property
    def selected_row(self):
        """
        Return the selected track
        """
        return self._selected_track

    @selected_row.setter
    def selected_row(self, value):
        self._selected_track=value

    @property
    def selected_event(self):
        """
        Return the selected event.
        """
        return self._selected


#!/usr/bin/python
# -*- coding: utf-8 -*-

from confapp import conf

from AnyQt           import _api
from AnyQt.QtWidgets import QToolBox
from AnyQt.QtWidgets import QFrame
from AnyQt.QtWidgets import QVBoxLayout
from AnyQt.QtWidgets import QHBoxLayout

from pyforms_gui.controls.control_base import ControlBase


class ControlToolBox(ControlBase):
    def init_form(self):
        self._form = QToolBox()
        super(ControlToolBox, self).init_form()

    # self.form.layout().setMargin(0)

    @property
    def value(self):
        return self._value

    @value.setter
    def value(self, value):
        ControlBase.label.fset(self, value);

        for item in range(self.form.count(), -1, -1): self.form.removeItem(item)

        for item in value:
            if isinstance(item, tuple):
                widget = QFrame(self.form);
                layout = QVBoxLayout();
                
                if _api.USED_API == _api.QT_API_PYQT5:
                    layout.setContentsMargins(0,0,0,0)
                elif _api.USED_API == _api.QT_API_PYQT4:
                    layout.setMargin(0)

                widget.setLayout(layout)

                for e in item[1]:
                    if isinstance(e, tuple):
                        hwidget = QFrame(self.form);
                        hlayout = QHBoxLayout();
                        
                        if _api.USED_API == _api.QT_API_PYQT5:
                            hlayout.setContentsMargins(0,0,0,0)
                        elif _api.USED_API == _api.QT_API_PYQT4:
                            hlayout.setMargin(0)
                            
                        hwidget.setLayout(hlayout)
                        for ee in e:
                            hlayout.addWidget(ee.form)
                        widget.layout().addWidget(hwidget)
                    else:
                        widget.layout().addWidget(e.form)
                self.form.addItem(widget, item[0])
            else:
                self.form.addItem(item.form, item.label)


    def set_item_enabled(self, index, enabled): 
        """
        Enable or disable an item
        """
        self.form.setItemEnabled(index, enabled)

    def is_item_enabled(self, index): 
        """
        Check if an item is enabled or disabled
        """
        return self.form.isItemEnabled(index)


# !/usr/bin/python
# -*- coding: utf-8 -*-

from AnyQt.QtGui import QColor
from .win_event import EventWindow

class Event(object):
	"""
	Class representing a time period \ event.
	"""

	def __init__(self, begin, end=30, title=None, lock=False, color=None, track=None, widget=None):
		"""
		Constructor
		
		:param int begin: First frame
		:param int end: Last frame
		:param str title: Event title
		:param bool lock: Flag to set the event lock
		:param AnyQt.QtGui.QColor color: Color of the event
		:param Track track: Track to witch the period will be added
		:param TimelineWidget widget: Parent timeline widget
		"""
		self._track  = track
		self._widget = widget
		self._title  = title
		self._lock 	 = lock
		self._begin  = begin
		self._end 	 = end
		self._end 	 = end
		self._color  = track.color if color is None else color
		
		track += self

	##########################################################################
	#### HELPERS/FUNCTIONS ###################################################
	##########################################################################


	def collide(self, xcoord, ycoord):
		"""
		Check if the x,y coordinate collides with the time period
		:param int xcoord: X coordinate in pixels.
		:param int ycoord: Y coordinate in pixels.
		:return: True if collide, False otherwise.
		"""
		return self.begin_coordinate <= xcoord <= self.end_coordinate and self.top_coordinate <= ycoord <= self.bottom_coordinate

	def in_range(self, start, end):
		"""
		Check if the period is within the limits [start;end]
		:param int start: Initial X coordinate
		:param int end: Final X coordinate
		:return: True if in range, False otherwise
		"""
		return start <= self.begin_coordinate and end >= self.end_coordinate or \
			   self.begin_coordinate <= start <= self.end_coordinate or \
			   self.begin_coordinate <= end <= self.end_coordinate

	def can_slide_begin(self, xcoord, ycoord):
		"""
		Checks if xcoord and ycoord pixel coordinate can slide the beginning of the period.
		:param int xcoord: X coordinate in pixels.
		:param int ycoord: Y coordinate in pixels.
		:return boolean: True if the event can be slided, false if not.
		"""
		# if locked does nothing
		if self._lock: return

		begin = int(round(self.begin_coordinate))
		end   = int(round(self.end_coordinate))
		return begin <= xcoord <= (begin+10) and \
			   self.top_coordinate <= ycoord <= self.bottom_coordinate and \
			   (xcoord-end)**2 > (xcoord-begin)**2

	def can_slide_end(self, x, y):
		"""
		Checks if x,y pixel coordinate can slide the end of the period.
		:param int x: X coordinate in pixels.
		:param int y: Y coordinate in pixels.
		:return boolean: True if the event can be slided, false if not.
		"""
		# if locked does nothing
		if self._lock: return

		begin = int(round(self.begin))
		end   = int(round(self.end))
		#check if the delta is not locked
		#check if the x is inside an range of 10 pixels
		#check if the y is within the boundaries of the delta
		return not self._lock and \
		   (end-10) <= x <= end and \
		   self.top_coordinate <= y <= self.bottom_coordinate and \
		   (x-end)**2 < (x-begin)**2

	def move_end(self, xdelta):
		"""
		Move the right edge of the period rectangle.
		:param int xdelta: X delta coordinate in pixels to move
		"""
		# Do nothing if locked
		if self._lock: return

		jump = xdelta / self._widget.scale

		# Do nothing if trying to go over the pther edge
		if (self.end_coordinate+jump) <= self.begin_coordinate and jump < 0:
			return

		# Increment accordingly
		self._end += jump

		# Minimum begin position is at 0
		if self._end > (self._widget.width() / self._widget.scale):
			self._end = (self._widget.width() / self._widget.scale)

	def move_begin(self, xcoord):
		"""
		Move the left edge of the period rectangle.
		:param int xdelta: X delta coordinate in pixels to move
		"""
		# Do nothing if locked
		if self._lock: return

		jump = xcoord/self._widget.scale

		# Do nothing if trying to go over the other edge
		if (self._begin+jump) >= self._end and jump > 0:
			return

		# Increment accordingly
		self._begin += jump

		# Minimum begin position is at 0
		if self._begin < 0: self._begin = 0



	def move(self, xdelta, ycoord):
		"""
		Move the event period more X and Y coordinates
		:param int xdelta: X delta to move in pixels
		:param int ycoord: Y coordinate to which the period should be moved
		"""
		# The period is locked, do nothing.
		if self._lock: return

		# if the new positions are within the 0 and maximum position,
		# then update the period positions
		if  (self.begin_coordinate + xdelta) >= 0 and \
			(self.end_coordinate + xdelta) <= self._widget.width():
			# update the positions
			delta = xdelta / self._widget.scale
			self._begin += delta
			self._end   += delta

		if not (self.track.bottom_coordinate <= ycoord <= self.track.top_coordinate):
			self.track = self._widget.find_track(ycoord)


	def draw(self, painter, showvalues=False):
		"""
		
		:param painter: 
		:param showvalues: 
		:return: 
		"""
		start, end 	 = self.begin_coordinate, self.end_coordinate
		transparency = 0.1 if self._lock else 0.5

		painter.setPen(QColor(0, 0, 0))
		painter.setOpacity(transparency)
		painter.drawRoundedRect( start, self.top_coordinate, end-start, self._widget.EVENT_HEIGHT, 3, 3)

		painter.setOpacity(1.0)
		painter.drawText(start + 3, self.top_coordinate + self._widget.EVENT_TITLE_TOP_POS, self.title)
		if showvalues:
			painter.drawText(
				start, self.top_coordinate + self._widget.EVENT_RANGE_TOP_POS,
				"[{};{}] delta:{}".format( int(self.begin_coordinate), int(self.end_coordinate), int(self.end_coordinate-self.begin_coordinate) )
			)

	def remove(self):
		"""
		Remove the period from the tracks
		"""
		self.track = None

	def open_properties(self):
		"""
		Open the properties window
		"""
		if hasattr(self, 'edit_form'):
			self.edit_form.update_form(self)
		else:
			self.edit_form = EventWindow(self._widget, self)

		self.edit_form.show()

	def __apply_properties_evt(self):
		"""
		Event called to apply the properties values in the Perido window
		:return:
		"""
		self._title = self.edit_form.comment
		self._begin = self.edit_form.begin
		self._end   = self.edit_form.end
		self._widget.repaint()
		self.edit_form.hide()

	##########################################################################
	#### PROPERTIES ##########################################################
	##########################################################################

	@property
	def title(self):
		return self._title

	@title.setter
	def title(self, value):
		self._title = value

	@property
	def lock(self):
		return self._lock

	@lock.setter
	def lock(self, value):
		self._lock = value

	@property
	def begin(self):
		return self._begin

	@begin.setter
	def begin(self, value):
		self._begin = value

	@property
	def end(self):
		return self._end

	@end.setter
	def end(self, value):
		self._end = value


	@property
	def track(self):
		"""
		:return: The current track
		"""
		return self._track

	@track.setter
	def track(self, value):
		"""
		Set the current track.
		:param value:
		"""
		# check if the new track is different from the previous.
		# if so update the tracks events lists
		if self._track != value:
			self._track -= self

			if value is not None:
				value += self

		self._track = value



	@property
	def top_coordinate(self):
		return self._track.top_coordinate + 2

	@property
	def bottom_coordinate(self):
		return self._track.bottom_coordinate - 2

	@property
	def begin_coordinate(self):
		return self._begin * self._widget.scale

	@begin_coordinate.setter
	def begin_coordinate(self, value):
		if self._lock: return
		self._begin = value / self._widget.scale
		if self._begin < 0: self._begin = 0

	@property
	def end_coordinate(self):
		return self._end * self._widget.scale

	@end_coordinate.setter
	def end_coordinate(self, value):
		if self._lock: return
		self._end = value / self._widget.scale
		if self._end > (self._widget.width() / self._widget.scale):
			self._end = (self._widget.width() / self._widget.scale)





	@property
	def color(self):
		return self._color

	@color.setter
	def color(self, value):
		self._color = QColor(value) if (type(value) == str) else value

	@property
	def bgrcolor(self):
		return self._color.blue(), self._color.green(), self._color.red()

	@property
	def properties(self):
		return ['P',
		        self._lock,
		        int(round(self._begin)),
		        int(round(self._end)),
		        self._title,
		        self._color.name(),
		        self.track]

	@properties.setter
	def properties(self, value):
		self._lock 	= value[1] == 'True'
		self._begin = int(value[2])
		self._end 	= int(value[3])
		self._title = value[4]
		self._color = QColor(value[5])
		self.track 	= int(value[6])

		self.checkNumberOfTracks()



#!/usr/bin/python
# -*- coding: utf-8 -*-

from pyforms_gui.controls.control_emptywidget import ControlEmptyWidget


class ControlDockWidget(ControlEmptyWidget):
	SIDE_LEFT = 'left'
	SIDE_RIGHT = 'right'
	SIDE_TOP = 'top'
	SIDE_BOTTOM = 'bottom'
	SIDE_DETACHED = 'detached'

	def __init__(self, *args, **kwargs):
		ControlEmptyWidget.__init__(self, *args, **kwargs)
		self.side = kwargs.get('side', 'left')
		self.order = kwargs.get('order',0)
		self.margin = kwargs.get('margin', 0)

		default = kwargs.get('default', None)
		if default is not None: self.value = default
		self._show = True

	@property
	def label(self):
		return self._label

	@label.setter
	def label(self, value):
		self._label = value
		if hasattr(self, 'dock'): self.dock.setWindowTitle(value)

	def save_form(self, data, path=None):
		data['side'] = self.side
		super(ControlDockWidget, self).save_form(data, path=None)

	def load_form(self, data):
		self.side = data['side']
		super(ControlDockWidget, self).load_form(data, path=None)

	def show(self):
		"""
		Show the control
		"""
		self._show = True
		if hasattr(self, 'dock'): self.dock.show()

	def hide(self):
		"""
		Hide the control
		"""
		self._show = False
		if hasattr(self, 'dock'): self.dock.hide()



#!/usr/bin/python
# -*- coding: utf-8 -*-

from confapp import conf

from AnyQt           import uic, QtCore


import pyforms_gui.utils.tools as tools
from pyforms_gui.controls.control_base import ControlBase


class ControlCheckBox(ControlBase):
    def init_form(self):
        control_path = tools.getFileInSameDirectory(__file__, "checkbox.ui")
        self._form = uic.loadUi(control_path)
        self._form.checkBox.setText(self._label)
        self._form.checkBox.stateChanged.connect(self.__checkedToggle)

        if self._value and self._value != '':
            self._form.checkBox.setCheckState(QtCore.Qt.Checked)
        else:
            self._form.checkBox.setCheckState(QtCore.Qt.Unchecked)

        if self.help: self.form.setToolTip(self.help)
        super(ControlCheckBox, self).init_form()

    def __checkedToggle(self):
        self.changed_event()

    def load_form(self, data, path=None):
        if 'value' in data:
            self._form.checkBox.setChecked(data['value'] == 'True' or data['value'] == True)

    def save_form(self, data, path=None):
        data['value'] = str(self._form.checkBox.isChecked())

    @property
    def value(self):
        return self._form.checkBox.isChecked()

    @value.setter
    def value(self, value):
        ControlBase.value.fset(self, value)
        self._form.checkBox.setChecked(value)
